import unittest
import sys
import os
import logging

sys.dont_write_bytecode = True
sys.path.insert(0, os.path.abspath(".."))
#sys.path.insert(0, os.path.abspath(os.path.join("..", "coshsh")))

import coshsh
from coshsh.generator import Generator
#from datasource import Datasource
from coshsh.host import Host
from coshsh.application import Application
from coshsh.monitoringdetail import MonitoringDetail

logger = logging.getLogger('coshsh')

class CoshshTest(unittest.TestCase):
    def setUp(self):
        self.generator = Generator()
        self.generator.setup_logging()
        self.hosts = {}
        self.applications = {}
        coshsh.application.Application.init_classes([
            os.path.join(os.path.dirname(__file__), '../recipes/default/classes'),
            os.path.join(os.path.dirname(__file__), 'recipes/test10/classes')])
        MonitoringDetail.init_classes([
            os.path.join(os.path.dirname(__file__), '../recipes/default/classes'),
            os.path.join(os.path.dirname(__file__), 'recipes/test10/classes')])
        pass
        row = ['drivelsrv', '11.120.9.10', 'Server', 'Red Hat 6.0', '', 'vs', '7x24', '2nd floor', 'ps']
        final_row = { }
        for (index, value) in enumerate(['host_name', 'address', 'type', 'os', 'hardware', 'virtual', 'notification_period', 'location', 'department']):
            final_row[value] = [None if row[index] == "" else row[index]][0]
        h = coshsh.host.Host(final_row)
        self.hosts[h.host_name] = h

        row = ['drivel', 'mysql', '', '', '', 'drivelsrv', '7x24']
        final_row = { }
        for (index, value) in enumerate(['name', 'type', 'component', 'version', 'patchlevel', 'host_name', 'check_period']):
            final_row[value] = [None if row[index] == "" else row[index]][0]
        a = coshsh.application.Application(final_row)
        #a.__init__(final_row)
        self.applications[a.fingerprint()] = a
        setattr(a, "host", self.hosts[a.host_name])

    def test_create_server(self):
        self.assert_("drivelsrv" in self.hosts)
        h = self.hosts["drivelsrv"]
        a = self.applications["drivelsrv+drivel+mysql"]
        self.assert_(hasattr(h, 'host_name'))
        self.assert_(h.host_name == 'drivelsrv')
        self.assert_(hasattr(a, 'host_name'))
        self.assert_(a.host_name == 'drivelsrv')
        self.assert_(hasattr(a, "port"))
        self.assert_(a.port == 3306)


    def test_create_server_alternative_port(self):
        self.assert_("drivelsrv" in self.hosts)
        h = self.hosts["drivelsrv"]
        a = self.applications["drivelsrv+drivel+mysql"]
        self.assert_(hasattr(h, 'host_name'))
        self.assert_(h.host_name == 'drivelsrv')
        self.assert_(hasattr(a, 'host_name'))
        self.assert_(a.host_name == 'drivelsrv')
        a.monitoring_details.append(coshsh.monitoringdetail.MonitoringDetail({
            "monitoring_type" : "PORT",
            "monitoring_0" : 10000}))
        a.resolve_monitoring_details()
        self.assert_(a.port == 10000)


if __name__ == '__main__':
    unittest.main()





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import coshsh
from coshsh.item import Item
from coshsh.templaterule import TemplateRule

class ContactGroup(coshsh.item.Item):

    id = 1 #0 is reserved for host (primary node for parents)
    my_type = 'contact_group'
    app_template = "app.tpl"

    template_rules = [
        coshsh.templaterule.TemplateRule(
            template="contactgroup",
            self_name="contactgroup",
            unique_attr="contactgroup_name", unique_config="contactgroup_%s",
        )
    ]


    def __init__(self, params={}):
        self.members = []
        super(ContactGroup, self).__init__(params)
        self.fingerprint = lambda s=self:s.__class__.fingerprint(params)

    @classmethod
    def fingerprint(self, params):
        return "%s" % (params["contactgroup_name"], )

    def __str__(self):
        return "contactgroup %s" % self.contactgroup_name




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sys
import os
import re
import inspect
import time
import logging
import errno
from jinja2 import FileSystemLoader, Environment, TemplateSyntaxError, TemplateNotFound
import coshsh
from coshsh.jinja2_extensions import is_re_match, filter_re_sub, filter_re_escape, filter_service, filter_custom_macros
from coshsh.item import Item
from coshsh.application import Application
from coshsh.hostgroup import HostGroup
from coshsh.monitoringdetail import MonitoringDetail
from coshsh.datasource import Datasource, DatasourceCorrupt, DatasourceNotReady, DatasourceNotAvailable, DatasourceNotCurrent
from coshsh.datarecipient import Datarecipient, DatarecipientCorrupt, DatarecipientNotReady, DatarecipientNotAvailable, DatarecipientNotCurrent
from coshsh.util import compare_attr, substenv

logger = logging.getLogger('coshsh')

class EmptyObject(object):
    pass


class RecipePidAlreadyRunning(Exception):
    # found another generator instance
    pass

class RecipePidNotWritable(Exception):
    # pid_dir is not writable
    pass

class RecipePidGarbage(Exception):
    # pid_file contains no integer
    pass


class Recipe(object):

    attributes_for_adapters = ["name", "force", "safe_output", "pid_dir", "pid_file", "templates_dir", "classes_dir", "objects_dir", "max_delta", "classes_path", "templates_path", "filter"]

    def __del__(self):
        pass
        # sys.path is invisible here, so this will fail
        #self.unset_recipe_sys_path()

    def __init__(self, **kwargs):
        for key in kwargs.iterkeys():
            if isinstance(kwargs[key], basestring):
                kwargs[key] = re.sub('%.*?%', substenv, kwargs[key])
        self.name = kwargs["name"]
        self.force = kwargs.get("force")
        self.safe_output = kwargs.get("safe_output")
        self.pid_dir = kwargs.get("pid_dir")
        if not self.pid_dir:
            if 'OMD_ROOT' in os.environ:
                self.pid_dir = os.path.join(os.environ['OMD_ROOT'], 'tmp/run')
            elif "cygwin" in sys.platform or "linux" in sys.platform:
                self.pid_dir = "/tmp"
            else:
                self.pid_dir = os.environ.get("%TEMP%", "C:/TEMP")

        self.pid_file = os.path.join(self.pid_dir, "coshsh.pid." + re.sub('[/\\\.]', '_', self.name))
        logger.info("recipe %s init" % self.name)
        self.templates_dir = kwargs.get("templates_dir")
        self.classes_dir = kwargs.get("classes_dir")
        self.max_delta = kwargs.get("max_delta", ())
        if isinstance(self.max_delta, str):
            if ":" in self.max_delta:
                self.max_delta = tuple(map(int, self.max_delta.split(":")))
            else:
                self.max_delta = tuple(map(int, (self.max_delta, self.max_delta)))
        self.my_jinja2_extensions = kwargs.get("my_jinja2_extensions", None)

        if 'OMD_ROOT' in os.environ:
            self.classes_path = [os.path.join(os.environ['OMD_ROOT'], 'share/coshsh/recipes/default/classes')]
        else:
            self.classes_path = [os.path.join(os.path.dirname(__file__), '../recipes/default/classes')]
        if self.classes_dir:
            for path in reversed([p.strip() for p in self.classes_dir.split(',')]):
                self.classes_path.insert(0, path)
        self.set_recipe_sys_path()
        if 'OMD_ROOT' in os.environ:
            self.templates_path = [os.path.join(os.environ['OMD_ROOT'], 'share/coshsh/recipes/default/templates')]
        else:
            self.templates_path = [os.path.join(os.path.dirname(__file__), '../recipes/default/templates')]
        if self.templates_dir:
            for path in reversed([p.strip() for p in self.templates_dir.split(',')]):
                self.templates_path.insert(0, path)
            logger.debug("recipe.templates_path reloaded %s" % ':'.join(self.templates_path))
        logger.info("recipe %s classes_dir %s" % (self.name, ','.join([os.path.abspath(p) for p in self.classes_path])))
        logger.info("recipe %s templates_dir %s" % (self.name, ','.join([os.path.abspath(p) for p in self.templates_path])))

        self.jinja2 = EmptyObject()
        setattr(self.jinja2, 'loader', FileSystemLoader(self.templates_path))
        setattr(self.jinja2, 'env', Environment(loader=self.jinja2.loader, extensions=['jinja2.ext.do']))
        self.jinja2.env.trim_blocks = True
        self.jinja2.env.tests['re_match'] = is_re_match
        self.jinja2.env.filters['re_sub'] = filter_re_sub
        self.jinja2.env.filters['re_escape'] = filter_re_escape
        self.jinja2.env.filters['service'] = filter_service
        self.jinja2.env.filters['custom_macros'] = filter_custom_macros

        if self.my_jinja2_extensions:
            for extension in [e.strip() for e in self.my_jinja2_extensions.split(",")]:
                imported = getattr(__import__("my_jinja2_extensions", fromlist=[extension]), extension)
                if extension.startswith("is_"):
                    self.jinja2.env.tests[extension.replace("is_", "")] = imported
                elif extension.startswith("filter_"):
                    self.jinja2.env.filters[extension.replace("filter_", "")] = imported
            
        self.datasources = []
        self.datarecipients = []

        self.objects = {
            'hosts': {},
            'hostgroups': {},
            'applications': {},
            'appdetails': {},
            'contacts': {},
            'contactgroups': {},
            'commands': {},
            'timeperiods': {},
            'dependencies': {},
            'bps': {},
        }
        
        self.old_objects = (0, 0)
        self.new_objects = (0, 0)

        self.datasource_filters = {}
        self.filter = kwargs.get("filter")
        if kwargs.get("filter"):
            for rule in kwargs.get("filter").split(','):
                match = re.match(r'(\w+)\((.*)\)', rule)
                if match:
                    self.datasource_filters[match.groups()[0].lower()] = match.groups()[1]
        self.init_class_cache()

        if kwargs.get("datasources"):
            self.datasource_names = [ds.lower() for ds in kwargs.get("datasources").split(",")]
        else:
            self.datasource_names = []
        if kwargs.get("objects_dir") and not kwargs.get("datarecipients"):
            self.objects_dir = kwargs["objects_dir"]
            logger.info("recipe %s objects_dir %s" % (self.name, os.path.abspath(self.objects_dir)))
            self.datarecipient_names = ["datarecipient_coshsh_default"]
        elif kwargs.get("objects_dir") and kwargs.get("datarecipients"):
            self.objects_dir = kwargs["objects_dir"]
            #logger.warn("recipe %s delete parameter objects_dir (use datarecipients instead)" % (self.name, ))
            self.datarecipient_names = [ds.lower() for ds in kwargs.get("datarecipients").split(",")]
        else:
            self.datarecipient_names = [ds.lower() for ds in kwargs.get("datarecipients").split(",")]
        # because this is allowed: datarecipients = >>>,SIMPLESAMPLE
        self.datarecipient_names = ['datarecipient_coshsh_default' if dr == '>>>' else dr for dr in self.datarecipient_names]
        if 'datarecipient_coshsh_default' in self.datarecipient_names:
            self.add_datarecipient(**dict([('type', 'datarecipient_coshsh_default'), ('name', 'datarecipient_coshsh_default'), ('objects_dir', self.objects_dir), ('max_delta', self.max_delta), ('safe_output', self.safe_output)]))

    def set_recipe_sys_path(self):
        for p in [p for p in reversed(self.classes_path) if os.path.exists(p) and os.path.isdir(p)]:
            sys.path.insert(0, os.path.abspath(p))

    def unset_recipe_sys_path(self):
        for p in [p for p in self.classes_path if os.path.exists(p) and os.path.isdir(p)]:
            sys.path.pop(0)

    def collect(self):
        data_valid = True
        for ds in self.datasources:
            filter = self.datasource_filters.get(ds.name)
            try:
                ds.open()
                pre_count = dict([(key, len(self.objects[key].keys())) for key in self.objects.keys()])
                pre_detail_count = sum([(len(obj.monitoring_details) if hasattr(obj, 'monitoring_details') else 99) for objs in [self.objects[key].values() for key in self.objects.keys()] for obj in objs], 0)
                ds.read(filter=filter, objects=self.objects, force=self.force)
                post_count = dict([(key, len(self.objects[key].keys())) for key in self.objects.keys()])
                post_detail_count = sum([(len(obj.monitoring_details) if hasattr(obj, 'monitoring_details') else 99) for objs in [self.objects[key].values() for key in self.objects.keys()] for obj in objs], 0)
                pre_count['details'] = pre_detail_count
                post_count['details'] = post_detail_count
                pre_count.update(dict.fromkeys([k for k in post_count if not k in pre_count], 0))
                chg_keys = [(key, post_count[key] - pre_count[key]) for key in set(pre_count.keys() + post_count.keys()) if post_count[key] != pre_count[key]]
                logger.info("recipe %s read from datasource %s %s" % (self.name, ds.name, ", ".join(["%d %s" % (k[1], k[0]) for k in chg_keys])))

                ds.close()
            except DatasourceNotCurrent:
                data_valid = False
                logger.info("datasource %s is is not current" % ds.name)
            except DatasourceNotReady:
                data_valid = False
                logger.info("datasource %s is busy" % ds.name)
            except DatasourceNotAvailable:
                data_valid = False
                logger.info("datasource %s is not available" % ds.name)
            except Exception, exp:
                data_valid = False
                logger.critical("datasource %s returns bad data (%s)" % (ds.name, exp))
            if not data_valid:
                logger.info("aborting collection phase") 
                return False

        for host in self.objects['hosts'].values():
            host.resolve_monitoring_details()
            host.create_templates()
            host.create_hostgroups()
            host.create_contacts()

        orphaned_applications = []
        for app in self.objects['applications'].values():
            try:
                setattr(app, 'host', self.objects['hosts'][app.host_name])
                app.resolve_monitoring_details()
                app.create_templates()
                app.create_servicegroups()
                app.create_contacts()
            except KeyError:
                logger.info("application %s %s refers to non-existing host %s" % (app.name, app.type, app.host_name))
                orphaned_applications.append(app.fingerprint())
        for oa in orphaned_applications:
            del self.objects['applications'][oa]

        # load the hostgroups-objects after application procession because
        # this allows modification of self.host.hostgroups
        # in application.wemustrepeat()
        for host in self.objects['hosts'].values():
            for hostgroup in host.hostgroups:
                try:
                    self.objects['hostgroups'][hostgroup].append(host.host_name)
                except Exception:
                    self.objects['hostgroups'][hostgroup] = []
                    self.objects['hostgroups'][hostgroup].append(host.host_name)
        for (hostgroup_name, members) in self.objects['hostgroups'].items():
            logger.debug("creating hostgroup %s" % hostgroup_name)
            self.objects['hostgroups'][hostgroup_name] = HostGroup({ "hostgroup_name" : hostgroup_name, "members" : members})
            self.objects['hostgroups'][hostgroup_name].create_templates()
            self.objects['hostgroups'][hostgroup_name].create_contacts()

        return True
 
    def render(self):
        template_cache = {}
        for host in self.objects['hosts'].values():
            host.render(template_cache, self.jinja2)
        for app in self.objects['applications'].values():
            # because of this __new__ construct the Item.searchpath is
            # not inherited. Needs to be done explicitely
            app.render(template_cache, self.jinja2)
        for cg in self.objects['contactgroups'].values():
            cg.render(template_cache, self.jinja2)
        for c in self.objects['contacts'].values():
            c.render(template_cache, self.jinja2)
        for hg in self.objects['hostgroups'].values():
            hg.render(template_cache, self.jinja2)
        # you can put anything in objects (Item class with own templaterules)
        for item in sum([self.objects[itype].values() for itype in self.objects if itype not in ['hosts', 'applications', 'contactgroups', 'contacts', 'hostgroups']], []):
            # first check hasattr, because somebody may accidentially
            # add objects which are not a subclass of Item.
            # (And such a stupid mistake crashes coshsh-cook)
            if hasattr(item, 'config_files') and not item.config_files:
                # has not been populated with content in the datasource
                # (like bmw appmon timeperiods)
                item.render(template_cache, self.jinja2)
            
    def count_before_objects(self):
        for datarecipient in self.datarecipients:
            datarecipient.count_before_objects()
        self.old_objects = (sum([dr.old_objects[0] for dr in self.datarecipients], 0), sum([dr.old_objects[1] for dr in self.datarecipients], 0))

    def count_after_objects(self):
        for datarecipient in self.datarecipients:
            datarecipient.count_after_objects()
        self.new_objects = (sum(0, [dr.new_objects[0] for dr in self.datarecipients]), sum(0, [dr.new_objects[1] for dr in self.datarecipients]))

    def cleanup_target_dir(self):
        for datarecipient in self.datarecipients:
            datarecipient.cleanup_target_dir()

    def prepare_target_dir(self):
        for datarecipient in self.datarecipients:
            datarecipient.prepare_target_dir()

    def output(self):
        for datarecipient in self.datarecipients:
            datarecipient.count_before_objects()
            datarecipient.load(None, self.objects)
            datarecipient.cleanup_target_dir()
            datarecipient.prepare_target_dir()
            datarecipient.output()

    def read(self):
        return self.objects


    def init_class_cache(self):
        Datasource.init_classes(self.classes_path)
        logger.debug("init Datasource classes (%d)" % len(Datasource.class_factory))
        Datarecipient.init_classes(self.classes_path)
        logger.debug("init Datarecipient classes (%d)" % len(Datarecipient.class_factory))
        Application.init_classes(self.classes_path)
        logger.debug("init Application classes (%d)" % len(Application.class_factory))
        MonitoringDetail.init_classes(self.classes_path)
        logger.debug("init MonitoringDetail classes (%d)" % len(MonitoringDetail.class_factory))

    def add_datasource(self, **kwargs):
        for key in [k for k in kwargs.iterkeys() if isinstance(kwargs[k], str)]:
            kwargs[key] = re.sub('%.*?%', substenv, kwargs[key])
        newcls = Datasource.get_class(kwargs)
        if newcls:
            for key in [attr for attr in self.attributes_for_adapters if hasattr(self, attr)]:
                kwargs['recipe_'+key] = getattr(self, key)
            datasource = newcls(**kwargs)
            self.datasources.append(datasource)

    def add_datarecipient(self, **kwargs):
        for key in [k for k in kwargs.iterkeys() if isinstance(kwargs[k], str)]:
            kwargs[key] = re.sub('%.*?%', substenv, kwargs[key])
        newcls = Datarecipient.get_class(kwargs)
        if newcls:
            for key in [attr for attr in self.attributes_for_adapters if hasattr(self, attr)]:
                kwargs['recipe_'+key] = getattr(self, key)
            datarecipient = newcls(**kwargs)
            self.datarecipients.append(datarecipient)

    def pid_protect(self):
        if os.path.exists(self.pid_file):
            if not os.access(self.pid_file, os.W_OK):
                raise RecipePidNotWritable
            try:
                pid = int(open(self.pid_file).read().strip())
            except ValueError:
                raise RecipePidGarbage
            try:
                os.kill(pid, 0)
            except OSError, (code, text):
                if code == errno.ESRCH:
                    # The pid doesn't exist, so remove the stale pidfile.
                    os.remove(self.pid_file)
                    logger.info('removing stale (pid %d) pidfile %s' % (pid, self.pid_file))
                else:
                    logger.info('failed to check status of process %d from pidfile %s: %s' % (pid, self.pid_file, text))
                    raise RecipePidGarbage
            else:
                logger.info('another instance seems to be running (pid %s), exiting' % pid)
                raise RecipePidAlreadyRunning
        else:
            if not os.access(self.pid_dir, os.W_OK):
                raise RecipePidNotWritable
        pid = str(os.getpid())
        try:
            file(self.pid_file, 'w').write(pid)
        except Exception:
            raise RecipePidNotWritable
        else:
            return self.pid_file

    def pid_remove(self):
        try:
            os.remove(self.pid_file)
        except Exception:
            pass




import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "KEYVALUES":
        return MonitoringDetailKeyvalues


class MonitoringDetailKeyvalues(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "generic"
    property_type = dict

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.dictionary = { }
        try:
            self.dictionary[params["monitoring_0"]] = params["monitoring_1"]
        except Exception:
            pass
        try:
            self.dictionary[params["monitoring_2"]] = params["monitoring_3"]
        except Exception:
            pass
        try:
            self.dictionary[params["monitoring_4"]] = params["monitoring_5"]
        except Exception:
            pass




import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging


sys.dont_write_bytecode = True

import coshsh
from coshsh.generator import Generator


class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test1", True)
        os.makedirs("./var/objects/test1")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging(scrnloglevel=logging.DEBUG)

    def tearDown(self):
        #shutil.rmtree("./var/objects/test1", True)
        print 

    def test_create_recipe_check_paths(self):
        self.print_header()
        self.generator.add_recipe(name='test4', **dict(self.config.items('recipe_TEST4')))
        return
        self.assert_(os.path.abspath(self.generator.recipes['test4'].classes_path[0]) == os.path.abspath('./recipes/test4/classes'))
        self.assert_(os.path.abspath(self.generator.recipes['test4'].templates_path[0]) == os.path.abspath('recipes/test4/templates'))
        self.assert_(os.path.abspath(self.generator.recipes['test4'].jinja2.loader.searchpath[0]) == os.path.abspath('recipes/test4/templates'))
  
        self.generator.add_recipe(name='test5', **dict(self.config.items('recipe_TEST5')))
        self.assert_(os.path.abspath(self.generator.recipes['test5'].classes_path[0]) == os.path.abspath('./recipes/test5/classes'))
        self.assert_(os.path.abspath(self.generator.recipes['test5'].templates_path[0]) == os.path.abspath('./recipes/test5/templates'))
        self.assert_(os.path.abspath(self.generator.recipes['test5'].jinja2.loader.searchpath[0]) == os.path.abspath('./recipes/test5/templates'))
  
        # did the jinja2 object get the self-written filters?
        self.assert_('re_match' in self.generator.recipes['test5'].jinja2.env.tests)
        self.assert_('service' in self.generator.recipes['test5'].jinja2.env.filters)

    def test_create_recipe_check_factories(self):
        self.print_header()
        self.generator.add_recipe(name='test4', **dict(self.config.items('recipe_TEST4')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        ds = coshsh.datasource.Datasource(**dict(cfg))
        self.assert_(hasattr(ds, 'only_the_test_simplesample'))
        self.config.set("datasource_CSVSAMPLE", "name", "csvsample")
        cfg = self.config.items("datasource_CSVSAMPLE")
        ds = coshsh.datasource.Datasource(**dict(cfg))
        self.assert_(ds.dir == "./recipes/test1/data")

    def test_create_recipe_check_factories_env(self):
        self.print_header()
        os.environ['COSHSHDIR'] = '/opt/coshsh'
        os.environ['ZISSSSSSCHDIR'] = '/opt/zisch'
        self.generator.add_recipe(name='test7', **dict(self.config.items('recipe_TEST7')))
        self.config.set("datasource_ENVDIRDS", "name", "envdirds")
        cfg = self.config.items("datasource_ENVDIRDS")
        ds = coshsh.datasource.Datasource(**dict(cfg))
        self.assert_(ds.dir == "/opt/coshsh/recipes/test7/data")
        self.assert_(self.generator.recipes['test7'].classes_path[0:2] == ['/opt/coshsh/recipes/test7/classes', '/opt/zisch/tmp'])

    def test_create_recipe_check_factories_read(self):
        self.print_header()
        self.generator.add_recipe(name='test4', **dict(self.config.items('recipe_TEST4')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        ds = coshsh.datasource.Datasource(**dict(cfg))
        self.assert_(hasattr(ds, 'only_the_test_simplesample'))
        objects = self.generator.recipes['test4'].objects
        ds.read(objects=objects)
        self.assert_(objects['hosts']['test_host_0'].my_host == True)
        self.assert_(objects['applications'].values()[0].test4_linux == True)
        self.assert_(objects['applications'].values()[1].test4_windows == True)


    def test_create_recipe_check_3factories_read(self):
        self.print_header()
        self.generator.add_recipe(name='test4a', **dict(self.config.items('recipe_TEST4A')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        ds = coshsh.datasource.Datasource(**dict(cfg))
        self.assert_(hasattr(ds, 'only_the_test_simplesample'))
        objects = self.generator.recipes['test4a'].objects
        ds.read(objects=objects)
        self.assert_(objects['hosts']['test_host_0'].my_host == True)
        self.assert_(objects['applications']['test_host_0+os+windows'].test4_windows == True)
        self.assert_(objects['applications']['test_host_0+os+red hat'].mycorp_linux == True)

    def test_create_recipe_check_factories_write(self):
        self.print_header()
        self.generator.add_recipe(name='test4', **dict(self.config.items('recipe_TEST4')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test4'].add_datasource(**dict(cfg))

        # remove target dir / create empty
        self.generator.recipes['test4'].count_before_objects()
        self.generator.recipes['test4'].cleanup_target_dir()

        self.generator.recipes['test4'].prepare_target_dir()
        # check target

        # read the datasources
        self.generator.recipes['test4'].collect()

        # for each host, application get the corresponding template files
        # get the template files and cache them in a struct owned by the recipe
        # resolve the templates and attach the result as config_files to host/app
        self.generator.recipes['test4'].render()
        self.assert_(hasattr(self.generator.recipes['test4'].objects['hosts']['test_host_0'], 'config_files'))
        self.assert_('host.cfg' in self.generator.recipes['test4'].objects['hosts']['test_host_0'].config_files)

        # write hosts/apps to the filesystem
        self.generator.recipes['test4'].output()
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_windows_default.cfg"))
        os_windows_default_cfg = open("var/objects/test1/dynamic/hosts/test_host_0/os_windows_default.cfg").read()
        self.assert_('os_windows_default_check_unittest' in os_windows_default_cfg)


    def test_create_recipe_check_factories_write2(self):
        self.print_header()
        self.generator.add_recipe(name='test4', **dict(self.config.items('recipe_TEST4B')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test4'].add_datasource(**dict(cfg))
        self.config.set("datarecipient_SIMPLESAMPLE2", "name", "simplesample")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE2")
        self.generator.recipes['test4'].add_datarecipient(**dict(cfg))


        # remove target dir / create empty
        self.generator.recipes['test4'].count_before_objects()
        self.generator.recipes['test4'].cleanup_target_dir()

        self.generator.recipes['test4'].prepare_target_dir()
        # check target

        # read the datasources
        self.generator.recipes['test4'].collect()
        
        # for each host, application get the corresponding template files
        # get the template files and cache them in a struct owned by the recipe
        # resolve the templates and attach the result as config_files to host/app
        self.generator.recipes['test4'].render()
        self.assert_(hasattr(self.generator.recipes['test4'].objects['hosts']['test_host_0'], 'config_files'))
        self.assert_('host.cfg' in self.generator.recipes['test4'].objects['hosts']['test_host_0'].config_files)

        # write hosts/apps to the filesystem
        self.generator.recipes['test4'].output()
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_windows_default.cfg"))
        os_windows_default_cfg = open("var/objects/test1/dynamic/hosts/test_host_0/os_windows_default.cfg").read()
        self.assert_('os_windows_default_check_unittest' in os_windows_default_cfg)

    def test_ds_handshake(self):
        self.print_header()
        self.generator.add_recipe(name='test8', **dict(self.config.items('recipe_TEST8')))
        self.config.set("datasource_HANDSH", "name", "handshake")
        cfg = self.config.items("datasource_HANDSH")
        ds = coshsh.datasource.Datasource(**dict(cfg))
        try:
            hosts, applications, contacts, contactgroups, appdetails, dependencies, bps = ds.read()
        except Exception, exp:
            pass
        self.assert_(exp.__class__.__name__ == "DatasourceNotCurrent")
        cfg = self.config.items("datasource_HANDSH")
        self.generator.recipes['test8'].add_datasource(**dict(cfg))
        coll_success = self.generator.recipes['test8'].collect()
        self.assert_(coll_success == False)

    def xtest_rebless_class(self):
        self.print_header()
        self.generator.add_recipe(name='test1', **dict(self.config.items('recipe_TEST1')))

        af = ApplicationFactory()
        print af.class_cache
        self.generator.recipes['test1'].init_class_cache()
        print "tete", Application.class_factory
        row = ['gms1', 'gearman-server', '', '', '', 'nagioscop001', '7x24']
        final_row = { }
        for (index, value) in enumerate(['name', 'type', 'component', 'version', 'patchlevel', 'host_name', 'check_period']):
            final_row[value] = [None if row[index] == "" else row[index]][0]
        a = Application(final_row)
        print a


if __name__ == '__main__':
    unittest.main()





import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging


sys.dont_write_bytecode = True

import coshsh
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.datarecipient import Datarecipient
from coshsh.application import Application

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test9", True)
        os.makedirs("./var/objects/test9")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging()

    def tearDown(self):
        #shutil.rmtree("./var/objects/test1", True)
        print 

    def test_factories(self):
        self.print_header()
        self.generator.add_recipe(name='test9', **dict(self.config.items('recipe_TEST9')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        ds = coshsh.datasource.Datasource(**dict(cfg))
        self.assert_(hasattr(ds, 'only_the_test_simplesample'))
        self.config.set("datarecipient_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE")
        ds = coshsh.datarecipient.Datarecipient(**dict(cfg))
        self.assert_(hasattr(ds, 'only_the_test_simplesample') and ds.only_the_test_simplesample == False)

    def test_create_recipe_check_factories(self):
        self.print_header()
        self.generator.add_recipe(name='test9', **dict(self.config.items('recipe_TEST9')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test9'].add_datasource(**dict(cfg))
        self.config.set("datarecipient_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE")
        self.generator.recipes['test9'].add_datarecipient(**dict(cfg))
        self.assert_(self.generator.recipes['test9'].datasources[0].only_the_test_simplesample == True)
        self.assert_(self.generator.recipes['test9'].datarecipients[0].only_the_test_simplesample == False)


    def test_create_recipe_fallback_datarecipient(self):
        self.print_header()
        self.generator.add_recipe(name='test9a', **dict(self.config.items('recipe_TEST9a')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test9a'].add_datasource(**dict(cfg))
        # there must be a recipient named "datarecipient_coshsh_default"
        self.assert_(self.generator.recipes['test9a'].datarecipients[0].name == "datarecipient_coshsh_default")
        # which must inherit the recipe's object_dir
        self.assert_(self.generator.recipes['test9a'].datarecipients[0].objects_dir == self.generator.recipes['test9a'].objects_dir)

    def test_create_recipe_fallback_datarecipient_write(self):
        self.print_header()
        self.generator.add_recipe(name='test9a', **dict(self.config.items('recipe_TEST9a')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test9a'].add_datasource(**dict(cfg))
        self.assert_(self.generator.recipes['test9a'].datarecipients[0].name == "datarecipient_coshsh_default")
        exporter = self.generator.recipes['test9a'].datarecipients[0]
        exporter.count_before_objects()
        self.assert_(exporter.old_objects == (0, 0))
        self.generator.recipes['test9a'].collect()
        # fill items.[cfgfiles]
        self.generator.recipes['test9a'].render()
        self.generator.recipes['test9a'].output()
        exporter.count_after_objects()
        self.assert_(exporter.new_objects == (1, 2))

    def test_mcid_recipient(self):
        self.print_header()
        recipe = {'classes_dir': './recipes/test9/classes', 'datarecipients': 'xpaas'}
        datasource = {'name': 'datasource', 'type': 'simplesample'}
        datareceiver = {'name': 'datarcv', 'type': 'xpaas', 'objects_dir': './var/objects/test9'}
        self.generator.add_recipe(name='recp', **recipe)
        self.generator.recipes['recp'].add_datasource(**datasource)
        self.generator.recipes['recp'].add_datarecipient(**datareceiver)
        self.generator.recipes['recp'].collect()
        mcid = 1
        for host in self.generator.recipes['recp'].objects['hosts'].values():
            setattr(host, "mcid", "%010d" % mcid)
            mcid += 1
        for app in self.generator.recipes['recp'].objects['applications'].values():
            setattr(app, "mcid", "%010d" % mcid)
            mcid += 1
        self.generator.recipes['recp'].render()
        self.generator.recipes['recp'].output()

    def test_create_recipe_collect(self):
        self.print_header()
        self.generator.add_recipe(name='test9', **dict(self.config.items('recipe_TEST9')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test9'].add_datasource(**dict(cfg))
        self.config.set("datarecipient_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE")
        self.generator.recipes['test9'].add_datarecipient(**dict(cfg))
        self.generator.recipes['test9'].collect()
        self.generator.recipes['test9'].output()




if __name__ == '__main__':
    unittest.main()





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "RAM":
        return MonitoringDetailRam


class MonitoringDetailRam(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "ram"
    property_type = str

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.warning = params["monitoring_0"]
        self.critical = params["monitoring_1"]

    def __str__(self):
        return "w:%s,c:%s" % (self.warning, self.critical)




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import coshsh


class HostGroup(coshsh.item.Item):

    id = 1 #0 is reserved for host (primary node for parents)
    template_rules = [
        coshsh.templaterule.TemplateRule(needsattr=None, 
            template="hostgroup", 
            self_name="hostgroup",
            unique_attr="hostgroup_name", unique_config="hostgroup_%s",
        ),
    ]


    def __init__(self, params={}):
        self.members = []
        self.contacts = []
        self.contactgroups = []
        self.templates = []
        superclass = super(HostGroup, self)
        superclass.__init__(params)
        

    def is_correct(self):
        return True


    def write_config(self, target_dir):
        my_target_dir = os.path.join(target_dir, "hostgroups", self.hostgroup_name)
        if not os.path.exists(my_target_dir):
            os.mkdir(my_target_dir)
        for file in self.config_files:
            content = self.config_files[file]
            with open(os.path.join(my_target_dir, file), "w") as f:
                f.write(content)


    def create_members(self):
        pass


    def create_contacts(self):
        pass


    def create_templates(self):
        pass




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

class TemplateRule:
    def __init__(self, needsattr=None, isattr=None, template=None, unique_attr="name", unique_config=None, self_name="application", suffix="cfg"):
        # This rule applies by default (needsattr=None) or if a certain
        # property exists
        self.needsattr = needsattr
        self.isattr = isattr
        self.template = template
        # Sometimes more than one configs are needed
        # This property is used to separate the application objects
        self.unique_attr = unique_attr 
        # The name of the config file which can contain %s
        self.unique_config = unique_config
        self.suffix = suffix
        self.self_name = self_name
    
    def __str__(self):
        return "Rule: needsattr=%s, isattr=%s, template=%s, unique_attr=%s, unique_config=%s, suffix=%s, self_name=%s" % (self.needsattr, self.isattr, self.template, self.unique_attr, self.unique_config, self.suffix, self.self_name)




import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "NAGIOS":
        return MonitoringDetailNagios


class MonitoringDetailNagios(coshsh.monitoringdetail.MonitoringDetail):
    property = "generic"
    property_type = str

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.attribute = params.get("monitoring_0", None)
        self.value = params.get("monitoring_1", None)





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "LOGIN":
        return MonitoringDetailLogin


class MonitoringDetailLogin(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "login"
    property_type = str

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.username = params["monitoring_0"]
        self.password = params["monitoring_1"]






import coshsh
from coshsh.application import Application
from coshsh.templaterule import TemplateRule
from coshsh.util import compare_attr

def __mi_ident__(params={}):
    if coshsh.util.compare_attr("type", params, "mysql"):
        return MySQL


class MySQL(coshsh.application.Application):
    template_rules = [
        coshsh.templaterule.TemplateRule(
            template="app_db_mysql_default",
            unique_config="app_db_mysql_%s_default",
        )
    ]

    def __init__(self, params={}):
        super(MySQL, self).__init__(params)
        self.access = "remote"
        self.port = 3306



#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# Copyright 2010-2012 Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import csv
import os
import re
import logging
from copy import copy
import coshsh
from coshsh.datasource import Datasource, DatasourceNotAvailable
from coshsh.host import Host
from coshsh.application import Application
from coshsh.contactgroup import ContactGroup
from coshsh.contact import Contact
from coshsh.monitoringdetail import MonitoringDetail
from coshsh.util import compare_attr

logger = logging.getLogger('coshsh')

def __ds_ident__(params={}):
    if coshsh.util.compare_attr("type", params, "csv"):
        return CsvFile

class CommentedFile:
    def __init__(self, f, commentstring="#"):
        self.f = f
        self.commentstring = commentstring
    def next(self):
        line = self.f.next()
        while line.startswith(self.commentstring):
            line = self.f.next()
        return line
    def __iter__(self):
        return self

class CsvFile(coshsh.datasource.Datasource):
    def __init__(self, **kwargs):
        super(self.__class__, self).__init__(**kwargs)
        self.name = kwargs["name"]
        self.dir = kwargs["dir"]
        self.objects = {}

    def open(self):
        logger.info('open datasource %s' % self.name)
        if not os.path.exists(self.dir):
            logger.error('csv dir %s does not exist' % self.dir)
            raise coshsh.datasource.DatasourceNotAvailable

    def read(self, filter=None, objects={}, force=False, **kwargs):
        self.objects = objects
        try:
            hostreader = csv.DictReader(CommentedFile(open(os.path.join(self.dir, self.name+'_hosts.csv'))))
            logger.info('read hosts from %s' % os.path.join(self.dir, self.name+'_hosts.csv'))
        except Exception, exp:
            print "except", exp
            hostreader = []
        # host_name,address,type,os,hardware,virtual,notification_period,location,department
        for row in hostreader:
            row["templates"] = ["generic-host"]
            for attr in [k for k in row.keys() if k in ['type', 'os', 'hardware', 'virtual']]:
                row[attr] = row[attr].lower()
            h = coshsh.host.Host(row)
            self.add('hosts', h)

        try:
            appreader = csv.DictReader(CommentedFile(open(os.path.join(self.dir, self.name+'_applications.csv'))))
            logger.info('read applications from %s' % os.path.join(self.dir, self.name+'_applications.csv'))
        except Exception:
            appreader = []
        resolvedrows = []
        # name,type,component,version,host_name,check_period
        for row in appreader:
            for attr in [k for k in row.keys() if k in ['name', 'type', 'component', 'version']]:
                row[attr] = row[attr].lower()
            if '[' in row['host_name'] or '*' in row['host_name']:
                # hostnames can be regular expressions
                matching_hosts = [h for h in self.objects['hosts'].keys() if re.match('^('+row['host_name']+')', h)]
                for host_name in matching_hosts:
                    row['host_name'] = host_name
                    resolvedrows.append(copy(row))
            else:
                resolvedrows.append(copy(row))
        for row in resolvedrows:
            if not "virtual" in row:
                try:
                    row["virtual"] = self.objects['hosts'][row["host_name"]].virtual
                except KeyError:
                    logger.error('host %s not found for application %s' % (row["host_name"], row["name"]))
            a = coshsh.application.Application(row)
            self.add('applications', a)

        try:
            appdetailreader = csv.DictReader(CommentedFile(open(os.path.join(self.dir, self.name+'_applicationdetails.csv'))))
            logger.info('read appdetails from %s' % os.path.join(self.dir, self.name+'_applicationdetails.csv'))
        except Exception:
            appdetailreader = []
        resolvedrows = []
        # host_name,application_name,application_type,monitoring_type,monitoring_0,monitoring_1,monitoring_2,monitoring_3,monitoring_4,monitoring_5
        for row in appdetailreader:
            if '[' in row['host_name'] or '*' in row['host_name']:
                # hostnames can be regular expressions
                matching_hosts = [h for h in self.objects['hosts'].keys() if re.match('^('+row['host_name']+')', h)]
                for host_name in matching_hosts:
                    row['host_name'] = host_name
                    resolvedrows.append(copy(row))
            else:
                resolvedrows.append(copy(row))
        for row in resolvedrows:
            for attr in [k for k in row.keys() if k in ['application_name', 'application_type', 'component', 'version']]:
                row[attr] = row[attr].lower()
            application_id = "%s+%s+%s" % (row["host_name"], row["application_name"], row["application_type"])
            detail = coshsh.monitoringdetail.MonitoringDetail(row)
            if application_id in self.objects['applications']:
                self.objects['applications'][application_id].monitoring_details.append(detail)
            elif row["host_name"] in self.objects['hosts']:
                self.objects['hosts'][row["host_name"]].monitoring_details.append(detail)
            else:
                logger.info("found a detail %s for an unknown application %s" % (detail, application_id))
                raise
        
        try:
            contactgroupreader = csv.DictReader(CommentedFile(open(os.path.join(self.dir, self.name+'_contactgroups.csv'))))
            logger.info('read contactgroups from %s' % os.path.join(self.dir, self.name+'_contactgroups.csv'))
        except Exception:
            contactgroupreader = []
        resolvedrows = []
        # host_name,application_name,application_type,groups
        for row in contactgroupreader:
            if '[' in row['host_name'] or '*' in row['host_name']:
                # hostnames can be regular expressions
                matching_hosts = [h for h in self.objects['hosts'].keys() if re.match('^('+row['host_name']+')', h)]
                for host_name in matching_hosts:
                    row['host_name'] = host_name
                    resolvedrows.append(copy(row))
            else:
                resolvedrows.append(copy(row))
        for row in resolvedrows:
            application_id = "%s+%s+%s" % (row["host_name"], row["application_name"], row["application_type"])
            for group in row["groups"].split(":"):
                if not self.find('contactgroups', group):
                    self.add('contactgroups', coshsh.contactgroup.ContactGroup({ 'contactgroup_name' : group }))
                if self.find('applications', application_id) and row["application_name"] == "os":
                    if not group in self.get('applications', application_id).contact_groups:
                        self.get('applications', application_id).contact_groups.append(group)
                    # OS contacts also are host's contacts
                    if not group in self.get('hosts', row["host_name"]).contact_groups:
                        self.get('hosts', row["host_name"]).contact_groups.append(group)
                elif self.find('applications', application_id):
                    if not group in self.get('applications', application_id).contact_groups:
                        self.get('applications', application_id).contact_groups.append(group)
                elif ("application_name" not in row or not row['application_name']) and self.find('hosts', row['host_name']):
                    if not group in self.get('hosts', row['host_name']).contact_groups:
                        self.get('hosts', row['host_name']).contact_groups.append(group)
                else:
                    logger.error('no such application %s for contactgroup %s' % (application_id, row['groups']))

        
        try:
            contactreader = csv.DictReader(CommentedFile(open(os.path.join(self.dir, self.name+'_contacts.csv'))))
            logger.info('read contacts from %s' % os.path.join(self.dir, self.name+'_contacts.csv'))
        except Exception:
            contactreader = []
        # name,type,address,userid,notification_period,groups
        for row in contactreader:
            c = coshsh.contact.Contact(row)
            if not self.find('contacts', c.fingerprint()):
                c.contactgroups.extend(row["groups"].split(":"))
                self.add('contacts', c)





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "FILESYSTEM":
        return MonitoringDetailFilesystem


class MonitoringDetailFilesystem(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "filesystems"
    property_type = list
    unique_attribute = "path"

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        if "monitoring_0" in params:
            self.path = params["monitoring_0"]
            self.warning = str(params.get("monitoring_1", "10"))
            self.critical = str(params.get("monitoring_2", "5"))
            self.units = params.get("monitoring_3", "%")
            self.optional = bool(params.get("monitoring_4", "0"))
            self.iwarning = str(params.get("monitoring_5", "0"))
            self.icritical = str(params.get("monitoring_6", "0"))
        else:
            self.path = params["path"]
            self.warning = str(params.get("warning", "10"))
            self.critical = str(params.get("critical", "5"))
            self.units = params.get("units", "%")
            self.optional = bool(params.get("optional", "0"))
        if not self.units:
            self.units = "%"

    def __str__(self):
        return "%s %s:%s:0:100" % (self.path, self.warning, self.critical)




import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging

print "sys.path", sys.path
import coshsh
from coshsh.contact import Contact
from coshsh.contactgroup import ContactGroup
from coshsh.host import Host
from coshsh.application import Application
from coshsh.hostgroup import HostGroup
from coshsh.generator import Generator

sys.dont_write_bytecode = True

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test1", True)
        os.makedirs("./var/objects/test1")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')

    def tearDown(self):
        #shutil.rmtree("./var/objects/test1", True)
        print

    def test_read_config(self):
        self.print_header()
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')

    def test_create_generator(self):
        self.print_header()
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging(scrnloglevel=logging.DEBUG)

    def xtest_create_generator_by_coshsh(self):
        self.print_header()
        self.generator = coshsh.Generator()
        self.generator.setup_logging(scrnloglevel=logging.DEBUG)

    def test_create_hostgroup(self):
        self.print_header()
        self.hostgroup = coshsh.hostgroup.HostGroup()
        self.assert_(self.hostgroup.members == [])

    def test_create_contactgroup(self):
        self.print_header()
        self.contactgroup = coshsh.contactgroup.ContactGroup()
        self.assert_(self.contactgroup.members == [])

    def test_create_contact(self):
        self.print_header()
        self.contact = coshsh.contact.Contact({"type": "WEB", "name": "sepp", "userid": "test", "notification_period": "5x8"})
        self.assert_(self.contact.contact_name == "test")
        self.assert_(self.contact.host_notification_period == "5x8")

    def test_create_host(self):
        self.print_header()
        self.host = coshsh.host.Host({"host_name": "test"})
        self.assert_(self.host.hostgroups == [])
        self.assert_(self.host.ports == [22])
        self.assert_(self.host.alias == "test")

    def test_create_application(self):
        self.print_header()
        self.application = coshsh.application.Application({"host_name": "test", "name": "shop", "type": "apache"})
        self.assert_(self.application.contact_groups == [])
        self.assert_(self.application.__class__.__name__ == "GenericApplication")

    def test_create_application(self):
        self.print_header()
        self.application = coshsh.application.Application({"host_name": "test", "name": "shop", "type": "apache"})
        self.assert_(self.application.contact_groups == [])
        print self.application.fingerprint()
        print self.application.__class__.__name__

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# Copyright 2010-2012 Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import re
import logging
import coshsh
from copy import copy
from coshsh.util import compare_attr
from coshsh.datasource import Datasource
from coshsh.host import Host
from coshsh.application import Application
from coshsh.contactgroup import ContactGroup
from coshsh.contact import Contact
from coshsh.monitoringdetail import MonitoringDetail

logger = logging.getLogger('coshsh')

def __ds_ident__(params={}):
    if coshsh.util.compare_attr("type", params, "simplesample"):
        return SimpleSample


class SimpleSample(coshsh.datasource.Datasource):
    def __init__(self, **kwargs):
        super(self.__class__, self).__init__(**kwargs)
        #self.name = kwargs["name"]
        self.dir = kwargs.get("dir", "/tmp")
        self.objects = {}

    def open(self):
        logger.info('open datasource %s' % self.name)
        return True

    def read(self, filter=None, objects={}, force=None, **kwargs):
        logger.info('read items from simplesample')
        #self.add('hosts', Host(...))




import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging
from logging import INFO, DEBUG

logger = logging.getLogger('coshsh')

sys.dont_write_bytecode = True

import coshsh
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.application import Application
from coshsh.monitoringdetail import MonitoringDetail


class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test6", True)
        os.makedirs("./var/objects/test6")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging(scrnloglevel=DEBUG)
        self.generator.add_recipe(name='test6', **dict(self.config.items('recipe_TEST6')))
        self.config.set("datasource_CSVDETAILS", "name", "test6")

    def tearDown(self):
        shutil.rmtree("./var/objects/test6", True)
        print 

    def test_detail_keyvalues(self):
        self.print_header()
        cfg = self.config.items("datasource_CSVDETAILS")
        objects = self.generator.recipes['test6'].objects
        ds = coshsh.datasource.Datasource(**dict(cfg))
        ds.read(objects=objects)
        app1 = objects['applications'].values()[0]
        app1.resolve_monitoring_details()
        app2 = objects['applications'].values()[1]
        app2.resolve_monitoring_details()
        # swap threshold via KEYVALUES detail
        self.assert_(app1.swap_warning == "15%")
        self.assert_(app1.swap_critical == "8%")
        # cron threshold via KEYVALUES detail
        self.assert_(app1.cron_warning == "30")
        self.assert_(app1.cron_critical == "100")
        # swap threshold via class os_linux
        self.assert_(app2.swap_warning == "5%")
        self.assert_(app2.swap_critical == "15%")
        # neither class detail nor csv detail
        self.assert_(not hasattr(app2, "cron_warning"))
        self.assert_(hasattr(app2, "thresholds"))
        self.assert_(hasattr(app2.thresholds, "cron_warning"))
        self.assert_(app2.thresholds.cron_warning == "31")

    def test_detail_url(self):
        self.print_header()
        oracle = coshsh.application.Application({
            'host_name': 'test',
            'name': 'test',
            'type': 'generic',
        })
        url = coshsh.monitoringdetail.MonitoringDetail({
            'host_name': 'test',
            'application_name': 'test',
            'application_type': 'generic',
            'monitoring_type': 'URL',
            'monitoring_0': 'oracle://dbadm:pass@dbsrv:1522/svc',
        })
        oracle.monitoring_details.append(url)
        oracle.resolve_monitoring_details()
        self.assert_(len(oracle.urls) == 1)
        # consol app_db_oracle class will call wemustrepeat() to create
        # a fake LOGIN-detail, so there is a oracle.username
        self.assert_(oracle.urls[0].username == 'dbadm')
        self.assert_(oracle.urls[0].password == 'pass')
        self.assert_(oracle.urls[0].hostname == 'dbsrv')
        self.assert_(oracle.urls[0].port == 1522)
        # will be without the / in the consol app_db_oracle class
        self.assert_(oracle.urls[0].path == '/svc')

    def test_detail_ram(self):
        coshsh.application.Application.init_classes([
            os.path.join(os.path.dirname(__file__), 'recipes/test6/classes'),
            os.path.join(os.path.dirname(__file__), '../recipes/default/classes')])
        coshsh.monitoringdetail.MonitoringDetail.init_classes([
            os.path.join(os.path.dirname(__file__), 'recipes/test6/classes'),
            os.path.join(os.path.dirname(__file__), '../recipes/default/classes')])
        self.print_header()
        opsys = coshsh.application.Application({'name': 'os', 'type': 'red hat 6.1'})
        ram = coshsh.monitoringdetail.MonitoringDetail({'application_name': 'os',
            'application_type': 'red hat 6.1',
            'monitoring_type': 'RAM',
            'monitoring_0': '80',
            'monitoring_1': '90',
        })
        opsys.monitoring_details.append(ram)
        for m in opsys.monitoring_details:
            print "detail", m
        opsys.resolve_monitoring_details()
        self.assert_(hasattr(opsys, 'ram'))
        self.assert_(opsys.ram.warning == '80')

    def test_detail_2url(self):
        self.print_header()
        cfg = self.config.items("datasource_CSVDETAILS")
        objects = self.generator.recipes['test6'].objects
        ds = coshsh.datasource.Datasource(**dict(cfg))
        ds.read(objects=objects)



        #coshsh.application.Application.init_classes([
        #    os.path.join(os.path.dirname(__file__), 'recipes/test6/classes'),
        #    os.path.join(os.path.dirname(__file__), '../recipes/default/classes')])
        #coshsh.monitoringdetail.MonitoringDetail.init_classes([
        #    os.path.join(os.path.dirname(__file__), 'recipes/test6/classes'),
        #    os.path.join(os.path.dirname(__file__), '../recipes/default/classes')])
        opsys = coshsh.application.Application({'host_name': 'test_host_0', 'name': 'testapp', 'type': 'webapp'})
        url1 = coshsh.monitoringdetail.MonitoringDetail({
            'host_name': 'test_host_0',
            'application_name': 'testapp',
            'application_type': 'webapp',
            'monitoring_type': 'URL',
            'monitoring_0': 'https://uzi75.schoggimaschin.com:5480/login.html',
            'monitoring_1': '10',
            'monitoring_2': '15',
        })
        opsys.monitoring_details.append(url1)
        url2 = coshsh.monitoringdetail.MonitoringDetail({
            'host_name': 'test_host_0',
            'application_name': 'testapp',
            'application_type': 'webapp',
            'monitoring_type': 'URL',
            'monitoring_0': 'https://uzi75.schoggimaschin.com/vsphere-client/?csp',
            'monitoring_1': '10',
            'monitoring_2': '15',
        })
        opsys.monitoring_details.append(url2)
        ds.add('applications', opsys)
        for m in opsys.monitoring_details:
            print "detail", m
        opsys.resolve_monitoring_details()
        self.assert_(hasattr(opsys, 'urls'))
        for u in opsys.urls:
            print "url is", u, u.__dict__
        shutil.rmtree("./var/objects/test6", True)
        os.makedirs("./var/objects/test6/dynamic")
        self.generator.recipes['test6'].collect()
        self.generator.recipes['test6'].render()
        self.generator.recipes['test6'].output()
        self.assert_(os.path.exists('var/objects/test6/dynamic/hosts/test_host_0/app_generic_web.cfg'))
        with open('var/objects/test6/dynamic/hosts/test_host_0/app_generic_web.cfg', 'r') as outfile:
            for line in outfile.read().split('\n'):
                print line


if __name__ == '__main__':
    unittest.main()





import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging


sys.dont_write_bytecode = True

import coshsh
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.application import Application

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging()

    def tearDown(self):
        pass

    def test_recipe_max_deltas_default(self):
        self.print_header()
        recipe = {'classes_dir': '/tmp', 'objects_dir': '/tmp', 'templates_dir': '/tmp', 'datasources': 'datasource' }
        datasource = {'name': 'datasource', 'type': 'simplesample'}
        self.generator.add_recipe(name='recp', **recipe)
        self.generator.recipes['recp'].add_datasource(**datasource)
        self.assert_(self.generator.recipes['recp'].max_delta == ())

    def test_recipe_max_deltas_simple(self):
        self.print_header()
        recipe = {'classes_dir': '/tmp', 'objects_dir': '/tmp', 'templates_dir': '/tmp', 'datasources': 'datasource', 'max_delta': '101' }
        datasource = {'name': 'datasource', 'type': 'simplesample'}
        self.generator.add_recipe(name='recp', **recipe)
        self.generator.recipes['recp'].add_datasource(**datasource)
        self.assert_(self.generator.recipes['recp'].max_delta == (101, 101))

    def test_recipe_max_deltas_double(self):
        self.print_header()
        recipe = {'classes_dir': '/tmp', 'objects_dir': '/tmp', 'templates_dir': '/tmp', 'datasources': 'datasource', 'max_delta': '101:202' }
        datasource = {'name': 'datasource', 'type': 'simplesample'}
        self.generator.add_recipe(name='recp', **recipe)
        self.generator.recipes['recp'].add_datasource(**datasource)
        self.assert_(self.generator.recipes['recp'].max_delta == (101, 202))

    def test_create_recipe_multiple_sources(self):
        self.print_header()
        self.generator.add_recipe(name='test10', **dict(self.config.items('recipe_TEST10')))
        self.config.set("datasource_CSV10.1", "name", "csv1")
        self.config.set("datasource_CSV10.2", "name", "csv2")
        self.config.set("datasource_CSV10.3", "name", "csv3")
        cfg = self.config.items("datasource_CSV10.1")
        self.generator.recipes['test10'].add_datasource(**dict(cfg))
        cfg = self.config.items("datasource_CSV10.2")
        self.generator.recipes['test10'].add_datasource(**dict(cfg))
        cfg = self.config.items("datasource_CSV10.3")
        self.generator.recipes['test10'].add_datasource(**dict(cfg))
        exit
        # remove target dir / create empty
        self.generator.recipes['test10'].count_before_objects()
        self.generator.recipes['test10'].cleanup_target_dir()

        self.generator.recipes['test10'].prepare_target_dir()
        # check target

        # read the datasources
        self.generator.recipes['test10'].collect()

        # for each host, application get the corresponding template files
        # get the template files and cache them in a struct owned by the recipe
        # resolve the templates and attach the result as config_files to host/app
        self.generator.recipes['test10'].render()
        self.assert_(hasattr(self.generator.recipes['test10'].objects['hosts']['test_host_0'], 'config_files'))
        self.assert_('host.cfg' in self.generator.recipes['test10'].objects['hosts']['test_host_0'].config_files)

        # write hosts/apps to the filesystem
        self.generator.recipes['test10'].output()
        self.assert_(os.path.exists("var/objects/test10/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test10/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test10/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test10/dynamic/hosts/test_host_1/os_windows_default.cfg"))
        os_windows_default_cfg = open("var/objects/test10/dynamic/hosts/test_host_1/os_windows_default.cfg").read()
        self.assert_('os_windows_default_check_' in os_windows_default_cfg)
        self.assert_(len(self.generator.recipes['test10'].objects['applications']['test_host_1+os+windows2k8r2'].filesystems) == 3)

if __name__ == '__main__':
    unittest.main()





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# Copyright 2010-2012 Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import re
import logging
import coshsh
from coshsh.host import Host
from coshsh.datasource import Datasource
from coshsh.application import Application
from coshsh.util import compare_attr

logger = logging.getLogger('coshsh')

def __ds_ident__(params={}):
    if coshsh.util.compare_attr("type", params, "simplesample"):
        return SimpleSample

class MyHost(coshsh.host.Host):
    def __init__(self, params={}):
        superclass = super(MyHost, self)
        superclass.__init__(params)
        self.my_host = True

class SimpleSample(coshsh.datasource.Datasource):
    class_only_the_test_simplesample = True
    def __init__(self, **kwargs):
        self.name = kwargs["name"]
        self.only_the_test_simplesample = True

    def read(self, filter=None, objects={}, **kwargs):
        logger.info('read items from simplesample')
        self.objects = objects
        hostdata = {
            'host_name': 'test_host_0',
            'address': '127.0.0.9',
            'type': 'test',
            'os': 'Red Hat 6.3',
            'hardware': 'Vmware',
            'virtual': 'vs',
            'notification_period': '7x24',
            'location': 'esxsrv10',
            'department': 'test',
        }
        self.add('hosts', MyHost(hostdata))
        appdata = {
            'name': 'os',
            'type': 'Red Hat',
            'component': '',
            'version': '6.3',
            'patchlevel': '',
            'host_name': 'test_host_0',
            'check_period': '7x24',
        }
        self.add('applications', coshsh.application.Application(appdata))
        appdata = {
            'name': 'os',
            'type': 'Windows',
            'component': '',
            'version': '2008',
            'patchlevel': 'R2',
            'host_name': 'test_host_0',
            'check_period': '7x24',
        }
        self.add('applications', coshsh.application.Application(appdata))



import unittest
import random
import re
import os
import sys
import shutil
import string
import tempfile
from optparse import OptionParser
import ConfigParser
import logging


sys.dont_write_bytecode = True

import coshsh
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.application import Application
from coshsh.util import substenv

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test1", True)
        os.makedirs("./var/objects/test1")
        shutil.rmtree("./var/tmp", True)
        os.makedirs("./var/tmp")
        recipe_configs = {}
        datasource_configs = {}
        datarecipient_configs = {}
        cookbook = ConfigParser.ConfigParser()
        cookbook.read("etc/coshsh.cfg")
        for ds in [section for section in cookbook.sections() if section.startswith('datarecipient_')]:
            datarecipient_configs[ds.replace("datarecipient_", "", 1).lower()] = cookbook.items(ds) + [('name', ds.replace("datarecipient_", "", 1).lower())]
        for ds in [section for section in cookbook.sections() if section.startswith('datasource_')]:
            datasource_configs[ds.replace("datasource_", "", 1).lower()] = cookbook.items(ds) + [('name', ds.replace("datasource_", "", 1).lower())]
        for recipe in [section for section in cookbook.sections() if section.startswith('recipe_')]:
            recipe_configs[recipe.replace("recipe_", "", 1).lower()] = cookbook.items(recipe) + [('name', recipe.replace("recipe_", "", 1).lower())]
        recipes = []
        if False:
            pass
            #recipes = [r.lower() for r in opts.default_recipe.split(",")]
        else:
            if "defaults" in cookbook.sections() and "recipes" in [c[0] for c in cookbook.items("defaults")]:
                recipes = [recipe.lower() for recipe in dict(cookbook.items("defaults"))["recipes"].split(",")]
            else:
                recipes = recipe_configs.keys()
        if "defaults" in cookbook.sections() and "log_dir" in [c[0] for c in cookbook.items("defaults")]:
            log_dir = dict(cookbook.items("defaults"))["log_dir"]
            log_dir = re.sub('%.*?%', substenv, log_dir)
        else:
            coshsh_home = os.path.join(os.path.dirname(__file__), '..', 'coshsh')
            os.environ['COSHSH_HOME'] = coshsh_home
            log_dir = os.path.join(os.environ['COSHSH_HOME'], "..")
        if "defaults" in cookbook.sections() and "pid_dir" in [c[0] for c in cookbook.items("defaults")]:
            pid_dir = dict(cookbook.items("defaults"))["pid_dir"]
            pid_dir = re.sub('%.*?%', substenv, pid_dir)
        else:
            pid_dir = os.path.join(os.environ['COSHSH_HOME'], "..")
        self.generator = coshsh.generator.Generator()
        for recipe in recipes:
            if recipe == 'test1':
                recipe_configs[recipe].append(('pid_dir', pid_dir))
                self.generator.add_recipe(**dict(recipe_configs[recipe]))



    def tearDown(self):
        if os.path.exists(self.generator.recipes['test1'].pid_file):
            os.remove(self.generator.recipes['test1'].pid_file)

    def test_run(self):
        self.print_header()
        self.generator.run()

    def test_pid_exists(self):
        self.print_header()
        pid_file = self.generator.recipes['test1'].pid_protect()
        self.assert_(os.path.exists(pid_file))

    def test_pid_exists(self):
        self.print_header()
        pid_file = self.generator.recipes['test1'].pid_protect()
        self.assert_(os.path.exists(pid_file))
        pid = open(self.generator.recipes['test1'].pid_file).read().strip()
        self.assert_(pid == str(os.getpid()))
        self.generator.recipes['test1'].pid_remove()
        self.assert_(not os.path.exists(pid_file))

    def test_pid_stale(self):
        self.print_header()
        while True:
            pid = random.randrange(1,50000)
            try:
                os.kill(pid, 0)
            except OSError, (code, text):
                # does not exist
                file(self.generator.recipes['test1'].pid_file, 'w').write(str(pid))
                print "stale pid is", open(self.generator.recipes['test1'].pid_file).read().strip()
                break
        pid_file = self.generator.recipes['test1'].pid_protect()
        self.assert_(os.path.exists(pid_file))
        pid = open(self.generator.recipes['test1'].pid_file).read().strip()
        print "my pid is", open(self.generator.recipes['test1'].pid_file).read().strip()
        self.assert_(pid == str(os.getpid()))

    def test_pid_blocked(self):
        self.print_header()
        while True:
            pid = random.randrange(1,50000)
            try:
                os.kill(pid, 0)
                file(self.generator.recipes['test1'].pid_file, 'w').write(str(pid))
                print "running pid is", open(self.generator.recipes['test1'].pid_file).read().strip()
                break
            except OSError, (code, text):
                # does not exist
                pass
        try:
            pid_file = self.generator.recipes['test1'].pid_protect()
        except Exception, exp:
            print "exp is ", exp.__class__.__name__
        self.assert_(exp.__class__.__name__ == "RecipePidAlreadyRunning")

    def test_pid_garbled(self):
        self.print_header()
        file(self.generator.recipes['test1'].pid_file, 'w').write("schlonz")
        self.generator.run()
        try:
            pid_file = self.generator.recipes['test1'].pid_protect()
        except Exception, exp:
            print "exp is ", exp.__class__.__name__
        self.assert_(exp.__class__.__name__ == "RecipePidGarbage")

    def test_pid_perms(self):
        self.print_header()
        #self.generator.recipes['test1'].pid_dir = "/"
        self.generator.recipes['test1'].pid_dir = os.path.join(os.getcwd(), 'hundsglumpvarreckts')
        os.mkdir(self.generator.recipes['test1'].pid_dir)
        self.generator.run()
        os.chmod(self.generator.recipes['test1'].pid_dir, 0)
        try:
            pid_file = self.generator.recipes['test1'].pid_protect()
            os.remove(self.generator.recipes['test1'].pid_file)
        except Exception, exp:
            print "exp is ", exp.__class__.__name__
        self.assert_(exp.__class__.__name__ == "RecipePidNotWritable")
        os.rmdir(self.generator.recipes['test1'].pid_dir)
        
if __name__ == '__main__':
    unittest.main()





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}): 
    if params["monitoring_type"] == "PROCESS":
        return MonitoringDetailProcess


class MonitoringDetailProcess(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "processes"
    property_type = list
    mandatory_fields = ["monitoring_0"]

    def __init__(self, params):
        try:
            self.monitoring_type = params["monitoring_type"]
            if "monitoring_0" not in params:
                logger.info("mandatory parameter monitoring_0 missing %s:%s:%s" % (params["host_name"], params["application_name"], params["application_type"]))
            self.name = params["monitoring_0"]
            self.warning = params.get("monitoring_1", "1:1")
            self.critical = params.get("monitoring_2", "1:")
            self.alias = params.get("monitoring_3", self.name)
        except Exception:
            pass

    def __str__(self):
        return "%s %s:%s:0:100" % (self.name, self.warning, self.critical)





import coshsh
from coshsh.application import Application
from coshsh.templaterule import TemplateRule
from coshsh.util import compare_attr

def __mi_ident__(params={}):
    if coshsh.util.compare_attr("type", params, ".*windows.*"):
        return Windows


class Windows(coshsh.application.Application):
    template_rules = [
        coshsh.templaterule.TemplateRule(needsattr=None, 
            template="os_windows_default"),
        coshsh.templaterule.TemplateRule(needsattr=None,
            template="os_windows_kaas"),
    ]

    def __new__(cls, params={}):
        return object.__new__(cls)

    def __init__(self, params):
        self.test4_windows = True



class Windows2008(Windows):
    pass


class Windows2003(Windows):
    pass


class WindowsXP(Windows):
    pass


class WindowsXPSP1(Windows):
    pass


class WindowsVista(Windows):
    pass


class Windows7(Windows):
    pass





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sys
import os
import imp
import inspect
import logging
from urlparse import urlparse
import coshsh
from coshsh.item import Item
from coshsh.application import Application

logger = logging.getLogger('coshsh')

class MonitoringDetailNotImplemented(Exception):
    pass


class MonitoringDetail(coshsh.item.Item):
    class_factory = []
    lower_columns = ['application_name', 'application_type']

    def __init__(self, params):
        #print "Detail init", self.__class__, self.__class__.__name__, len(self.__class__.class_factory)
        if self.__class__.__name__ == "MonitoringDetail":
            for c in self.__class__.lower_columns:
                try:
                    params[c] = params[c].lower()
                except Exception:
                    if c in params:
                        params[c] = None
            newcls = self.__class__.get_class(params)
            if newcls:
                self.__class__ = newcls
                self.__init__(params)
            else:
                logger.info("monitoring detail of type %s for host %s / appl %s had a problem" % (params["monitoring_type"], params["host_name"], params["application_name"]))
                raise MonitoringDetailNotImplemented
        else:
            pass

    @classmethod
    def init_classes(cls, classpath):
        sys.dont_write_bytecode = True
        for p in [p for p in reversed(classpath) if os.path.exists(p) and os.path.isdir(p)]:
            for module, path in [(item, p) for item in os.listdir(p) if item[-3:] == ".py" and item.startswith('detail_')]:
                try:
                    path = os.path.abspath(path)
                    fp, filename, data = imp.find_module(module.replace('.py', ''), [path])
                    toplevel = imp.load_source(module.replace(".py", ""), filename)
                    for cl in inspect.getmembers(toplevel, inspect.isfunction):
                        if cl[0] ==  "__detail_ident__":
                            cls.class_factory.append([path, module, cl[1]])
                except Exception, e:
                    print e
                finally:
                    if fp:
                        fp.close()


    @classmethod
    def get_class(cls, params={}):
        #print "getclass from cache", cls, cls.__name__, len(cls.class_factory)
        for path, module, class_func in cls.class_factory:
            try:
                #print "get_class trys", path, module, class_func
                newcls = class_func(params)
                #print "get_class says", newcls
                if newcls:
                    return newcls
            except Exception:
                pass
        logger.debug("found no matching class for this monitoring detail %s" % params)
        return None





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

# maybe in a later release there will be something like
# from coshsh.host import Host
# so that older classes can be reused




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import logging
import coshsh
from coshsh.item import Item
from coshsh.templaterule import TemplateRule

logger = logging.getLogger('coshsh')

translations = (
    (u'\N{LATIN SMALL LETTER SHARP S}', u'ss'),
    (u'\N{LATIN SMALL LETTER O WITH DIAERESIS}', u'oe'),
    (u'\N{LATIN SMALL LETTER U WITH DIAERESIS}', u'ue'),
    (u'\N{LATIN CAPITAL LETTER A WITH DIAERESIS}', u'Ae'),
    (u'\N{LATIN CAPITAL LETTER O WITH DIAERESIS}', u'Oe'),
    (u'\N{LATIN CAPITAL LETTER U WITH DIAERESIS}', u'Ue'),
    # et cetera
)


class Contact(coshsh.item.Item):

    id = 1 #0 is reserved for host (primary node for parents)
    my_type = 'contact'
    app_template = "app.tpl"

    template_rules = [
        coshsh.templaterule.TemplateRule(
            template="contact",
            self_name="contact",
            unique_attr="contact_name", unique_config="contact_%s",
        )
    ]

    def __init__(self, params={}):
        self.email = None
        self.pager = None
        self.address1 = None
        self.address2 = None
        self.address3 = None
        self.address4 = None
        self.address5 = None
        self.address6 = None
        self.can_submit_commands = False
        self.contactgroups = []
        super(Contact, self).__init__(params)
        self.fingerprint = lambda s=self:s.__class__.fingerprint(params)
        if not hasattr(self, 'host_notification_period') or not self.host_notification_period:
            self.host_notification_period = self.notification_period
            logger.debug('no column host_notificatin_period found use notification_period instead')
        if not hasattr(self, 'service_notification_period') or not self.service_notification_period:
            self.service_notification_period = self.notification_period
            logger.debug('no column service_notificatin_period found use notification_period instead')
        try:
            for from_str, to_str in translations:
                self.name = self.name.replace(from_str, to_str)
            if self.type.startswith("WEB"):
                self.contact_name = self.userid
                if self.type == "WEBREADWRITE":
                    self.can_submit_commands = True
                self.service_notification_options = "n"
                self.host_notification_options = "n"
                self.service_notification_commands = ["notify_by_nothing"]
                self.host_notification_commands = ["notify_by_nothing"]
            elif self.type == "SMS":
                self.contact_name = "sms_" + self.name + "_" + self.notification_period.replace("/", "_")
                self.pager = self.address
                if not hasattr(self, "service_notification_options"):
                    setattr(self, "service_notification_options", "w,c,u,r,f")
                if not hasattr(self, "host_notification_options"):
                    setattr(self, "host_notification_options", "d,u,r,f")
                self.service_notification_commands = ["service-notify-by-sms"]
                self.host_notification_commands = ["host-notify-by-sms"]
            elif self.type == "PHONE":
                self.contact_name = "phone_" + self.name + "_" + self.notification_period.replace("/", "_")
                self.pager = self.address
            elif self.type == "MAIL":
                self.contact_name = "mail_" + self.name + "_" + self.notification_period.replace("/", "_")
                self.email = self.address
                if not hasattr(self, "service_notification_options"):
                    setattr(self, "service_notification_options", "w,c,u,r,f")
                if not hasattr(self, "host_notification_options"):
                    setattr(self, "host_notification_options", "d,u,r,f")
                self.service_notification_commands = ["service-notify-by-email"]
                self.host_notification_commands = ["host-notify-by-email"]
            else:
                self.contact_name = "unknown_" + self.name + "_" + self.notification_period.replace("/", "_")
        except Exception:
            self.contact_name = "failed_user"
            raise


    @classmethod
    def fingerprint(self, params):
        return "+".join([unicode(params.get(a, "")) for a in ["name", "type", "address", "userid"]])

    def __str__(self):
        fipri = " ".join([unicode(getattr(self, a, "")) for a in ["name", "type", "address", "userid"]])
        grps = ",".join(self.contactgroups)
        return unicode("contact %s groups (%s)" % (fipri, grps))





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "LOGINSNMPV3":
        return MonitoringDetailLoginSNMPV3


class MonitoringDetailLoginSNMPV3(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "loginsnmpv3"
    property_type = str

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.securityname = params.get("monitoring_0", None)
        self.authprotocol = params.get("monitoring_1", None)
        self.authkey = params.get("monitoring_2", None)
        self.privprotocol = params.get("monitoring_3", None)
        self.privkey = params.get("monitoring_4", None)
        self.context = params.get("monitoring_5", None)
        # noAuthNoPriv|authNoPriv|authPriv
        if not self.authkey and not self.privkey:
            self.securitylevel = "noAuthNoPriv"
        elif self.authkey and not self.privkey:
            self.securitylevel = "AuthNoPriv"
        elif self.authkey and self.privkey:
            self.securitylevel = "AuthPriv"




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os

class Dependency(object):

    def __init__(self, params={}):
        self.host_name = params["host_name"]
        self.parent_host_name = params["parent_host_name"]
        



import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "LOGINSNMPV2":
        return MonitoringDetailLoginSNMPV2


class MonitoringDetailLoginSNMPV2(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "loginsnmpv2"
    property_type = str

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.community = params.get("monitoring_0", "public") or "_none_"
        if self.community.startswith("v1:"):
            self.protocol = 1
            self.community = self.community.split(":", 1)[1]
        elif self.community.startswith("v2:"):
            self.protocol = 2
            self.community = self.community.split(":", 1)[1]
        else:
            self.protocol = 2

    def __str__(self):
        return "SNMP v2/1 community: %s" % (self.community)



import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging


sys.dont_write_bytecode = True
print __file__
sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
print sys.path

import coshsh
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.datarecipient import Datarecipient
from coshsh.application import Application

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test12", True)
        os.makedirs("./var/objects/test12")
        shutil.rmtree("./var/objects/test1", True)
        os.makedirs("./var/objects/test1")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging()

    def tearDown(self):
        shutil.rmtree("./var/objects/test12", True)
        shutil.rmtree("./var/objects/test1", True)
        print 

    def test_create_recipe_hand_down(self):
        attributes_for_adapters = ["name", "force", "safe_output", "pid_dir", "pid_file", "templates_dir", "classes_dir", "max_delta", "classes_path", "templates_path", "filter", "objects_dir"]
        self.print_header()
        self.generator.add_recipe(name='test12', **dict(self.config.items('recipe_TEST12')))

        self.config.set("datasource_CSV10.1", "name", "csv1")
        self.config.set("datasource_CSV10.2", "name", "csv2")
        self.config.set("datasource_CSV10.3", "name", "csv3")
        cfg = self.config.items("datasource_CSV10.1")
        self.generator.recipes['test12'].add_datasource(**dict(cfg))
        cfg = self.config.items("datasource_CSV10.2")
        self.generator.recipes['test12'].add_datasource(**dict(cfg))
        cfg = self.config.items("datasource_CSV10.3")
        self.generator.recipes['test12'].add_datasource(**dict(cfg))

        self.config.set("datarecipient_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE")
        self.generator.recipes['test12'].add_datarecipient(**dict(cfg))

        self.config.set("datarecipient_SIMPLESAMPLE2", "name", "simplesample2")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE2")
        self.generator.recipes['test12'].add_datarecipient(**dict(cfg))

        self.config.set("datarecipient_DEFAULT", "name", "default")
        cfg = self.config.items("datarecipient_DEFAULT")
        self.generator.recipes['test12'].add_datarecipient(**dict(cfg))

        dr_simplesample = [dr for dr in self.generator.recipes['test12'].datarecipients if dr.name == 'simplesample'][0]
        self.assert_(dr_simplesample.objects_dir == "/tmp")
        self.assert_(dr_simplesample.recipe_objects_dir == self.generator.recipes['test12'].objects_dir)
        dr_simplesample2 = [dr for dr in self.generator.recipes['test12'].datarecipients if dr.name == 'simplesample2'][0]
        self.assert_(dr_simplesample2.objects_dir == "./var/objects/test1")
        self.assert_(dr_simplesample2.recipe_objects_dir == self.generator.recipes['test12'].objects_dir)
        dr_default = [dr for dr in self.generator.recipes['test12'].datarecipients if dr.name == 'default'][0]
        self.assert_(dr_default.objects_dir == "./var/objects/test12")
        self.assert_(dr_default.recipe_objects_dir == self.generator.recipes['test12'].objects_dir)

        self.generator.recipes['test12'].collect()
        self.generator.recipes['test12'].render()
        self.generator.recipes['test12'].output()
        # written by datarecipient_coshsh_default
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts/test_host_1/os_windows_default.cfg"))
        os_windows_default_cfg = open("var/objects/test12/dynamic/hosts/test_host_1/os_windows_default.cfg").read()
        self.assert_('os_windows_default_check_unittest' in os_windows_default_cfg)
        # written by simplesample2 which has it's own objects_dir
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_1/os_windows_default.cfg"))

    def test_create_recipe_hand_down_implicit_default_dr(self):
        attributes_for_adapters = ["name", "force", "safe_output", "pid_dir", "pid_file", "templates_dir", "classes_dir", "max_delta", "classes_path", "templates_path", "filter", "objects_dir"]
        self.print_header()
        self.generator.add_recipe(name='test12a', **dict(self.config.items('recipe_TEST12a')))

        self.config.set("datasource_CSV10.1", "name", "csv1")
        self.config.set("datasource_CSV10.2", "name", "csv2")
        self.config.set("datasource_CSV10.3", "name", "csv3")
        cfg = self.config.items("datasource_CSV10.1")
        self.generator.recipes['test12a'].add_datasource(**dict(cfg))
        cfg = self.config.items("datasource_CSV10.2")
        self.generator.recipes['test12a'].add_datasource(**dict(cfg))
        cfg = self.config.items("datasource_CSV10.3")
        self.generator.recipes['test12a'].add_datasource(**dict(cfg))

        self.config.set("datarecipient_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE")
        self.generator.recipes['test12a'].add_datarecipient(**dict(cfg))

        self.config.set("datarecipient_SIMPLESAMPLE2", "name", "simplesample2")
        cfg = self.config.items("datarecipient_SIMPLESAMPLE2")
        self.generator.recipes['test12a'].add_datarecipient(**dict(cfg))

        dr_simplesample = [dr for dr in self.generator.recipes['test12a'].datarecipients if dr.name == 'simplesample'][0]
        self.assert_(dr_simplesample.objects_dir == "/tmp")
        self.assert_(dr_simplesample.recipe_objects_dir == self.generator.recipes['test12a'].objects_dir)
        dr_simplesample2 = [dr for dr in self.generator.recipes['test12a'].datarecipients if dr.name == 'simplesample2'][0]
        self.assert_(dr_simplesample2.objects_dir == "./var/objects/test1")
        self.assert_(dr_simplesample2.recipe_objects_dir == self.generator.recipes['test12a'].objects_dir)
        dr_default = [dr for dr in self.generator.recipes['test12a'].datarecipients if dr.name == 'datarecipient_coshsh_default'][0]
        self.assert_(dr_default.objects_dir == "./var/objects/test12")
        self.assert_(dr_default.recipe_objects_dir == self.generator.recipes['test12a'].objects_dir)

        self.generator.recipes['test12a'].collect()
        self.generator.recipes['test12a'].render()
        self.generator.recipes['test12a'].output()
        # written by datarecipient_coshsh_default
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test12/dynamic/hosts/test_host_1/os_windows_default.cfg"))
        os_windows_default_cfg = open("var/objects/test12/dynamic/hosts/test_host_1/os_windows_default.cfg").read()
        self.assert_('os_windows_default_check_unittest' in os_windows_default_cfg)
        # written by simplesample2 which has it's own objects_dir
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_1/os_windows_default.cfg"))


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sys
import os
import re
import imp
import inspect
import logging
import coshsh
from coshsh.util import compare_attr, substenv

logger = logging.getLogger('coshsh')

class DatarecipientNotImplemented(Exception):
    pass

class DatarecipientNotReady(Exception):
    # datarecipient is currently being updated
    pass

class DatarecipientNotCurrent(Exception):
    # datarecipients was not updated lately.
    # it makes no sense to continue.
    pass

class DatarecipientNotAvailable(Exception):
    pass

class DatarecipientCorrupt(Exception):
    pass


class Datarecipient(object):

    my_type = 'datarecipient'
    class_factory = []

    def __init__(self, **params):
        #print "datarecipientinit with", self.__class__
        for key in [k for k in params if k.startswith("recipe_")]:
            setattr(self, key, params[key])
        for key in params.iterkeys():
            if isinstance(params[key], basestring):
                params[key] = re.sub('%.*?%', substenv, params[key])
        if self.__class__ == Datarecipient:
            #print "generic ds", params
            newcls = self.__class__.get_class(params)
            if newcls:
                #print "i rebless anon datarecipient to", newcls, params
                self.__class__ = newcls
                self.__init__(**params)
            else:
                logger.critical('datarecipient for %s is not implemented' % params)
                #print "i raise DatarecipientNotImplemented"
                raise DatarecipientNotImplemented
        else:
            setattr(self, 'name', params["name"])
            self.objects = {}
            pass

    def load(self, filter=None, objects={}):
        logger.info('load items to %s' % (self.name, ))
        self.objects = objects

    def get(self, objtype, fingerprint):
        try:
            return self.objects[objtype][fingerprint]
        except Exception:
            # should be None
            return 'i do not exist. no. no!'

    def getall(self, objtype):
        try:
            return self.objects[objtype].values()
        except Exception:
            return []

    def find(self, objtype, fingerprint):
        return objtype in self.objects and fingerprint in self.objects[objtype]

    def item_write_config(self, obj, dynamic_dir, objtype):
        my_target_dir = os.path.join(dynamic_dir, objtype)
        if not os.path.exists(my_target_dir):
            os.makedirs(my_target_dir)
        for file in obj.config_files:
            content = obj.config_files[file]
            with open(os.path.join(my_target_dir, file), "w") as f:
                f.write(content)

    def output(self, filter=None):
        for hostgroup in self.objects['hostgroups'].values():
            self.item_write_config(hostgroup, self.dynamic_dir, "hostgroups")
        for host in self.objects['hosts'].values():
            self.item_write_config(host, self.dynamic_dir, os.path.join("hosts", host.host_name))
        for app in self.objects['applications'].values():
            self.item_write_config(app, self.dynamic_dir, os.path.join("hosts", app.host_name))
        for cg in self.objects['contactgroups'].values():
            self.item_write_config(cg, self.dynamic_dir, "contactgroups")
        for c in self.objects['contacts'].values():
            self.item_write_config(c, self.dynamic_dir, "contacts")

    def count_objects(self):
        try:
            hosts = len([name for name in os.listdir(os.path.join(self.dynamic_dir, 'hosts')) if os.path.isdir(os.path.join(self.dynamic_dir, 'hosts', name))])
            apps = len([app for host in os.listdir(os.path.join(self.dynamic_dir, 'hosts')) if os.path.isdir(os.path.join(self.dynamic_dir, 'hosts', host)) for app in os.listdir(os.path.join(self.dynamic_dir, 'hosts', host)) if app != 'host.cfg'])
            return (hosts, apps)
        except Exception:
            return (0, 0)

    def count_before_objects(self):
        self.old_objects = self.count_objects()

    def count_after_objects(self):
        self.new_objects = self.count_objects()

    def prepare_target_dir(self):
        pass

    def cleanup_target_dir(self):
        pass

    @classmethod
    def init_classes(cls, classpath):
        sys.dont_write_bytecode = True
        for p in [p for p in reversed(classpath) if os.path.exists(p) and os.path.isdir(p)]:
            for module, path in [(item, p) for item in os.listdir(p) if item[-3:] == ".py" and item.startswith('datarecipient_')]:
                try:
                    #print "try dr", module, path
                    path = os.path.abspath(path)
                    fp, filename, data = imp.find_module(module.replace('.py', ''), [path])
                    toplevel = imp.load_source(module.replace(".py", ""), filename)
                    for cl in inspect.getmembers(toplevel, inspect.isfunction):
                        if cl[0] ==  "__ds_ident__":
                            cls.class_factory.append([path, module, cl[1]])
                except Exception, exp:
                    logger.critical("could not load datarecipient %s from %s: %s" % (module, path, exp))
                finally:
                    if fp:
                        fp.close()


    @classmethod
    def get_class(cls, params={}):
        #print "get_classhoho", cls, len(cls.class_factory), cls.class_factory
        for path, module, class_func in cls.class_factory:
            try:
                #print "try", path, module, class_func
                newcls = class_func(params)
                if newcls:
                    return newcls
            except Exception ,exp:
                print "Datarecipient.get_class exception", exp
                pass
        logger.debug("found no matching class for this datarecipient %s" % params)







import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "INTERFACE":
        return MonitoringDetailInterface


class MonitoringDetailInterface(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "interfaces"
    property_type = list

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.name = params.get("monitoring_0", None)





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import re
import locale
import logging
from jinja2 import FileSystemLoader, Environment, TemplateSyntaxError, TemplateNotFound
from copy import copy, deepcopy

logger = logging.getLogger('coshsh')

class EmptyObject(object):
    pass


class Item(object):
    template_cache = {}

    @classmethod
    def reload_template_path(cls):
        loader = FileSystemLoader(cls.templates_path)
        cls.env = Environment(loader=loader)
        cls.env.trim_blocks = True

    def __init__(self, params={}):
        #print "Item.__init__(", self.__class__.__name__
        cls = self.__class__
        self.id = cls.id
        cls.id += 1

        self.log = logger

        for key in params:
            #print "set key", self.__class__.__name__, key
            setattr(self, key, params[key])

        if not hasattr(self, "monitoring_details"):
            # if not pre-set through the class
            self.monitoring_details = []
        else:
            setattr(self, "monitoring_details", list(self.__class__.monitoring_details))
        self.config_files = {}

    def clone(self):
        """ Return a copy of the item, but give him a new id """
        cls = self.__class__
        i = cls({})# Dummy item but with it's own running properties
        save_id = i.id
        new_obj = copy(self)
        new_obj.id = save_id
        return new_obj

    def write_config(self, target_dir):
        my_target_dir = os.path.join(target_dir, "hosts", self.host_name)
        for file in self.config_files:
            content = self.config_files[file]
            with open(os.path.join(my_target_dir, file), "w") as f:
                f.write(content)

    def resolve_monitoring_details(self):
        details = [d for d in self.monitoring_details]
        for detail in details:
            property = detail.__class__.property
            if property == "generic":
                if detail.__class__.property_type == dict:
                    for key in detail.dictionary:
                        if key:
                            if ":" in key:
                                dictname, key = key.split(":")
                                try:
                                    setattr(getattr(self, dictname), key, detail.dictionary[dictname + ":" + key])
                                except Exception:
                                    setattr(self, dictname, EmptyObject())
                                    setattr(getattr(self, dictname), key, detail.dictionary[dictname + ":" + key])
                            else:
                                setattr(self, key, detail.dictionary[key])
                else:
                    setattr(self, detail.attribute, detail.value)
            else:
                if detail.__class__.property_type == list:
                    if not hasattr(self, property):
                        setattr(self, property, [])
                    if hasattr(detail.__class__, "unique_attribute"):
                        # from the details remove an existing detail
                        # - which is of this class
                        # - which has the same unique_attr
                        if [o for o in getattr(self, property) if o.__class__ == detail.__class__ and getattr(o, detail.__class__.unique_attribute) == getattr(detail, detail.__class__.unique_attribute)]:
                            old = getattr(self, property)
                            new = [o for o in old if ((o.__class__ != detail.__class__) or (getattr(o, detail.__class__.unique_attribute) != getattr(detail, detail.__class__.unique_attribute)))]
                            new.append(detail)
                            setattr(self, property, new)
                        #print "res", self.host.host_name, detail.path
                        else:
                            getattr(self, property).append(detail)
                    else:
                        if hasattr(detail.__class__, "property_attr"):
                            getattr(self, property).append(getattr(detail, getattr(detail.__class__, "property_attr")))
                        else:
                            getattr(self, property).append(detail)
                else:
                    if getattr(detail.__class__, 'property_flat', False):
                        # ex. MonitoringDetailRole: appl.role == str instead of appl.role.role == str
                        setattr(self, property, getattr(detail, property))
                    else:
                        setattr(self, property, detail)
            # This detail has been resolved. Maybe we run resolve_monitoring_details
            # later again, we don't want to repeat ourselves.
            self.monitoring_details.remove(detail)
        if details:
            self.wemustrepeat()
        # example: if we have self.ports
        # and self.ports[0] has an inside property ports
        # and self has d default property port (set in __init__)
        # replace the self.port by self.ports[0].port
        for one_property in [detail.__class__.property.rstrip('s') for detail in details if detail.__class__.property_type == list and not hasattr(detail.__class__, "unique_attribute") and not hasattr(detail.__class__, "property_attr") and detail.__class__.property.endswith('s') and hasattr(self, detail.__class__.property.rstrip('s'))]:
            if hasattr(getattr(self, one_property + 's')[0], one_property):
                setattr(self, one_property, getattr(getattr(self, one_property + 's')[0], one_property))

    def wemustrepeat(self):
        """
        This method is called by some classes if different monitoring_details need to interact.
        Ex. (username/password was set in a LOGIN detail and/or a URL detail
        """
        pass

    def pythonize(self):
        if hasattr(self, "templates"):
            self.templates = self.templates.split(',')
        if hasattr(self, "contactgroups"):
            self.contactgroups = self.contactgroups.split(',')
        if hasattr(self, "contact_groups"): #! contacts have contact!_!groups
            self.contact_groups = self.contact_groups.split(',')
        if hasattr(self, "contacts"):
            self.contacts = self.contacts.split(',')
        if hasattr(self, "hostgroups"):
            self.hostgroups = self.hostgroups.split(',')
        if hasattr(self, "servicegroups"):
            self.servicegroups = self.servicegroups.split(',')
        if hasattr(self, "members"):
            self.members = self.members.split(',')
        if hasattr(self, "parents"):
            self.parents = self.parents.split(',')
        if hasattr(self, "host_notification_commands"):
            self.host_notification_commands = self.host_notification_commands.split(',')
        if hasattr(self, "service_notification_commands"):
            self.service_notification_commands = self.service_notification_commands.split(',')

    def depythonize(self):
        if hasattr(self, "templates"):
            self.templates = ",".join(self.templates)
        if hasattr(self, "contactgroups"):
            self.contactgroups = ",".join(sorted(self.contactgroups, cmp=locale.strcoll))
        if hasattr(self, "contact_groups"):
            self.contact_groups = ",".join(sorted(self.contact_groups, cmp=locale.strcoll))
        if hasattr(self, "contacts"):
            self.contacts = ",".join(sorted(self.contacts, cmp=locale.strcoll))
        if hasattr(self, "hostgroups"):
            self.hostgroups = ",".join(sorted(self.hostgroups, cmp=locale.strcoll))
        if hasattr(self, "servicegroups"):
            self.servicegroups = ",".join(sorted(self.servicegroups, cmp=locale.strcoll))
        if hasattr(self, "members"):
            self.members = ",".join(sorted(self.members, cmp=locale.strcoll))
        if hasattr(self, "parents"):
            self.parents = ",".join(sorted(self.parents, cmp=locale.strcoll))
        if hasattr(self, "host_notification_commands"):
            self.host_notification_commands = ",".join(sorted(self.host_notification_commands, cmp=locale.strcoll))
        if hasattr(self, "service_notification_commands"):
            self.service_notification_commands = ",".join(sorted(self.service_notification_commands, cmp=locale.strcoll))

    def render_cfg_template(self, jinja2, template_cache, name, output_name, suffix, **kwargs):
        try:
            if not name in template_cache:
                template_cache[name] = jinja2.env.get_template(name + ".tpl")
                logger.info("load template " + name)
        except TemplateSyntaxError as e:
            logger.critical("%s template %s has an error in line %d: %s" % (self.__class__.__name__, name, e.lineno, e.message))
        except TemplateNotFound:
            logger.error("cannot find template " + name)
        except Exception as exp:
            logger.critical("error in template %s (%s,%s)" (name, exp.__class__.__name__, exp))

        if name in template_cache:
            # transform hostgroups, contacts, etc. from list to string
            self.depythonize()
            try:
                self.config_files[output_name + "." + suffix] = template_cache[name].render(kwargs)
            except Exception as exp:
                if hasattr(self, "fingerprint"):
                    logger.critical("render exception in template %s for %s %s: %s" % (name, self, self.fingerprint(), exp))
                else:
                    logger.critical("render exception in template %s for %s: %s" % (name, self, exp))
            # transform hostgroups, contacts, etc. back to lists
            self.pythonize()

    def render(self, template_cache, jinja2):
        for rule in self.template_rules:
            render_this = False
            try:
                if not rule.needsattr:
                    # TemplateRule(template="os_solaris_default")
                    render_this = True
    
                elif (hasattr(self, rule.needsattr) and rule.isattr == None):
                    # TemplateRule(needsattr=None,template="os_solaris_default")
                    render_this = True
    
                elif hasattr(self, rule.needsattr) and not isinstance (getattr(self, rule.needsattr), list) and ((getattr(self, rule.needsattr) == rule.isattr) or re.match(rule.isattr, getattr(self, rule.needsattr))):
                    # TemplateRule(needsattr="cluster", isattr="veritas",
                    #     template="os_solaris_cluster_veritas")
                    render_this = True
    
                elif hasattr(self, rule.needsattr) and isinstance (getattr(self, rule.needsattr), list) and [elem for elem in getattr(self, rule.needsattr) if (elem == rule.isattr or re.match(rule.isattr, elem))]:
                    # TemplateRule(needsattr="hostgroups",
                    #     isattr="cluster_solaris_veritas.*",
                    #     template="os_solaris_cluster_veritas")
                    render_this = True
                    
                elif hasattr(self, rule.needsattr) and isinstance (getattr(self, rule.needsattr), list):
                    pass
            except Exception:
                logger.critical("error in %s template rules. please check %s" % (self.__class__.__name__, rule))

            if render_this:
                if rule.unique_config and isinstance(rule.unique_attr, basestring) and hasattr(self, rule.unique_attr):
                    self.render_cfg_template(jinja2, template_cache, rule.template, rule.unique_config % getattr(self, rule.unique_attr), rule.suffix, **dict([(rule.self_name, self)]))
                elif rule.unique_config and isinstance(rule.unique_attr, list) and reduce(lambda x, y: x and y, [hasattr(self, ua) for ua in rule.unique_attr]):
                    self.render_cfg_template(jinja2, template_cache, rule.template, rule.unique_config % tuple([getattr(self, a) for a in rule.unique_attr]), rule.suffix, **dict([(rule.self_name, self)]))
                else:
                    self.render_cfg_template(jinja2, template_cache, rule.template, rule.template, rule.suffix, **dict([(rule.self_name, self)]))

    def fingerprint(self):
        try:
            return "%s+%s+%s" % (self.host_name, self.name, self.type)
        except: pass
        try:
            return "%s+%s+%s" % (self.host_name, self.application_name, self.application_type)
        except: pass
        try:
            return "%s" % (self.host_name, )
        except: pass
        raise "impossible fingerprint"




import coshsh
from coshsh.application import Application
from coshsh.templaterule import TemplateRule
from coshsh.util import compare_attr

def __mi_ident__(params={}):
    if coshsh.util.compare_attr("type", params, ".*webapp.*"):
        return AppWeb


class AppWeb(coshsh.application.Application):
    template_rules = [
        coshsh.templaterule.TemplateRule(needsattr=None, 
            template="app_generic_web"),
    ]




import ConfigParser

class CoshshConfigParser(ConfigParser.ConfigParser, object):

    def read(self, files):
        super(self.__class__, self).read(files)
        for section in self._sections.values():
            if "isa" in section.keys() and section["isa"] in self._sections:
                    for key in self._sections[section["isa"]]:
                        if not key in section and not key == "isa":
                            section[key] = self._sections[section["isa"]][key]





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "TAG":
        return MonitoringDetailTag


class MonitoringDetailTag(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "tags"
    property_type = list
    property_flat = True
    property_attr = "tag" # application.tags will be a list of property.tag

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.tag = params["monitoring_0"]




import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import logging
import pprint


sys.dont_write_bytecode = True

import coshsh
from coshsh.configparser import CoshshConfigParser
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.application import Application

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        self.config = coshsh.configparser.CoshshConfigParser()
        self.config.read('etc/coshsh2.cfg')
        pp = pprint.PrettyPrinter(indent=4)
        #pp.pprint(self.config._sections.values())
                
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging()

    def tearDown(self):
        pass

    def test_inheritance(self):
        self.print_header()
        recipe = {'classes_dir': '/tmp', 'objects_dir': '/tmp', 'templates_dir': '/tmp', 'datasources': 'datasource' }
        self.generator.add_recipe(name='cust', **dict(self.config.items("recipe_cust")))
        self.generator.add_recipe(name='cust1', **dict(self.config.items("recipe_cust1")))
        self.generator.add_recipe(name='cust2', **dict(self.config.items("recipe_cust2")))
        self.generator.add_recipe(name='cust3', **dict(self.config.items("recipe_cust3")))
        self.assert_(self.generator.recipes['cust'].datasource_names == ['csv10.1', 'csv10.2', 'csv10.3'])
        self.assert_(self.generator.recipes['cust'].datasource_filters['simplesample'] == 'fff')
        self.assert_(self.generator.recipes['cust1'].datasource_names == ['csv10.1', 'csv10.2', 'csv10.3'])
        self.assert_(self.generator.recipes['cust1'].datasource_filters['simplesample'] == 'fff1')
        self.assert_(self.generator.recipes['cust2'].datasource_names == ['csv10.1', 'csv10.2', 'csv10.3'])
        self.assert_(self.generator.recipes['cust2'].datasource_filters['simplesample'] == 'fff2')
        self.assert_(self.generator.recipes['cust3'].datasource_names == ['csv10.1', 'csv10.2', 'csv10.3'])
        self.assert_(self.generator.recipes['cust3'].datasource_filters['simplesample'] == 'fff3')
        self.assert_('./recipes/test12/classes' in self.generator.recipes['cust3'].classes_path)


if __name__ == '__main__':
    unittest.main()





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import coshsh
from coshsh.item import Item
from coshsh.templaterule import TemplateRule


class Host(coshsh.item.Item):

    id = 1 #0 is reserved for host (primary node for parents)
    template_rules = [
        coshsh.templaterule.TemplateRule(needsattr=None, 
            template="host", 
            self_name="host",
        ),
    ]
    lower_columns = ['address', 'type', 'os', 'hardware', 'virtual', 'location', 'department']

    def __init__(self, params={}):
        for c in self.__class__.lower_columns:
            try:
                params[c] = params[c].lower()
            except Exception:
                if c in params:
                    params[c] = None
        self.hostgroups = []
        self.contacts = []
        self.contact_groups = []
        self.templates = []
        self.ports = [22] # can be changed with a PORT detail
        super(Host, self).__init__(params)
        self.alias = getattr(self, 'alias', self.host_name)
        self.fingerprint = lambda s=self:s.__class__.fingerprint(params)

    @classmethod
    def fingerprint(self, params):
        return "%s" % (params["host_name"], )

    def is_correct(self):
        return hasattr(self.host_name) and self.host_name != None

    def create_hostgroups(self):
        pass

    def create_contacts(self):
        pass

    def create_templates(self):
        pass




import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging


sys.dont_write_bytecode = True
sys.path.insert(0, os.path.abspath(".."))
sys.path.insert(0, os.path.abspath(os.path.join("..", "coshsh")))

import coshsh
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.application import Application

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test1", True)
        os.makedirs("./var/objects/test1")
        shutil.rmtree("./var/log", True)
        os.makedirs("./var/log")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging(logfile="zishsh.log", logdir="./var/log", scrnloglevel=logging.DEBUG)

    def tearDown(self):
        #shutil.rmtree("./var/objects/test1", True)
        print 

    def test_log(self):
        logger = logging.getLogger('coshsh')
        logger.warn("i warn you")
        logger.info("i inform you")
        logger.debug("i spam you")
        self.assert_(os.path.exists("./var/log/zishsh.log"))

    def test_write(self):
        self.print_header()
        self.generator.add_recipe(name='test4', **dict(self.config.items('recipe_TEST4')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test4'].add_datasource(**dict(cfg))

        # remove target dir / create empty
        self.generator.recipes['test4'].count_before_objects()
        self.generator.recipes['test4'].cleanup_target_dir()

        self.generator.recipes['test4'].prepare_target_dir()
        # check target

        # read the datasources
        self.generator.recipes['test4'].collect()

        # for each host, application get the corresponding template files
        # get the template files and cache them in a struct owned by the recipe
        # resolve the templates and attach the result as config_files to host/app
        self.generator.recipes['test4'].render()
        self.assert_(hasattr(self.generator.recipes['test4'].objects['hosts']['test_host_0'], 'config_files'))
        self.assert_('host.cfg' in self.generator.recipes['test4'].objects['hosts']['test_host_0'].config_files)

        # write hosts/apps to the filesystem
        self.generator.recipes['test4'].output()
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_linux_default.cfg"))
        self.assert_(os.path.exists("var/objects/test1/dynamic/hosts/test_host_0/os_windows_default.cfg"))
        os_windows_default_cfg = open("var/objects/test1/dynamic/hosts/test_host_0/os_windows_default.cfg").read()
        self.assert_('os_windows_default_check_unittest' in os_windows_default_cfg)
        self.assert_(os.path.exists("./var/log/zishsh.log"))


if __name__ == '__main__':
    unittest.main()





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "SOCKET":
        return MonitoringDetailSocket


class MonitoringDetailSocket(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "socket"
    property_type = str

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.socket = params["monitoring_0"]
        # Thresholds are shown only as an example. In this case they
        # indicate response times
        self.warning = params.get("monitoring_1", "1")
        self.critical = params.get("monitoring_2", "10")

    def __str__(self):
        return "%s" % (self.socket,)




import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging
import subprocess


sys.dont_write_bytecode = True

import coshsh
from coshsh.generator import Generator

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test1", True)
        os.makedirs("./var/objects/test1")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging()

    def tearDown(self):
        #shutil.rmtree("./var/objects/test1", True)
        pass 


    def test_create_template_tree(self):
        self.print_header()
        os.makedirs("./var/objects/test1/static")
        self.assert_(os.path.exists("var/objects/test1/static"))
        self.assert_(os.path.exists("../bin/coshsh-create-template-tree"))
        #subprocess.call(["../bin/coshsh-create-template-tree", "--cookbook", "etc/coshsh.cfg", "--recipe", "test4", "--template", "os_windows_fs"], shell=True)
        subprocess.call("../bin/coshsh-create-template-tree --cookbook etc/coshsh.cfg --recipe test4 --template os_windows_fs", shell=True)
        self.assert_(os.path.exists("var/objects/test1/static/service_templates"))
        self.assert_(os.path.exists("var/objects/test1/static/service_templates/os_windows_fs.cfg"))
        self.assert_(os.path.exists("var/objects/test1/static/service_templates/os_windows.cfg"))



if __name__ == '__main__':
    unittest.main()





import unittest
import os
import sys
import shutil
import string
from optparse import OptionParser
import ConfigParser
import logging


sys.dont_write_bytecode = True

import coshsh
from coshsh.generator import Generator
from coshsh.datasource import Datasource
from coshsh.datarecipient import Datarecipient
from coshsh.application import Application

class CoshshTest(unittest.TestCase):
    def print_header(self):
        print "#" * 80 + "\n" + "#" + " " * 78 + "#"
        print "#" + string.center(self.id(), 78) + "#"
        print "#" + " " * 78 + "#\n" + "#" * 80 + "\n"

    def setUp(self):
        shutil.rmtree("./var/objects/test11", True)
        os.makedirs("./var/objects/test11")
        self.config = ConfigParser.ConfigParser()
        self.config.read('etc/coshsh.cfg')
        self.generator = coshsh.generator.Generator()
        self.generator.setup_logging()

    def tearDown(self):
        shutil.rmtree("./var/objects/test11", True)
        print 

    def test_create_recipe_collect(self):
        self.print_header()
        self.generator.add_recipe(name='test11', **dict(self.config.items('recipe_TEST11')))
        self.config.set("datasource_SIMPLESAMPLE", "name", "simplesample")
        cfg = self.config.items("datasource_SIMPLESAMPLE")
        self.generator.recipes['test11'].add_datasource(**dict(cfg))
        #self.config.set("datarecipient_SIMPLESAMPLE", "name", "simplesample")
        #cfg = self.config.items("datarecipient_SIMPLESAMPLE")
        #self.generator.recipes['test11'].add_datarecipient(**dict(cfg))
        self.generator.recipes['test11'].collect()
        self.generator.recipes['test11'].render()
        self.generator.recipes['test11'].output()
        self.assert_(os.path.exists('var/objects/test11/dynamic/hosts/test_host_0/nrpe_os_windows_fs.conf'))
        self.assert_(os.path.exists('var/objects/test11/dynamic/hosts/test_host_0/os_windows_fs.cfg'))


if __name__ == '__main__':
    unittest.main()





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import time
import re
import copy
import os
from UserDict import DictMixin


class odict(DictMixin):
    """
    Copied from 
    http://code.activestate.com/recipes/496761-a-more-clean-implementation-for-ordered-dictionary/
    Thanks! I use the odict class for generator.recipes
    """
    def __init__(self):
        self._keys = []
        self._data = {}

    def __setitem__(self, key, value):
        if key not in self._data:
            self._keys.append(key)
        self._data[key] = value

    def __getitem__(self, key):
        return self._data[key]

    def __delitem__(self, key):
        del self._data[key]
        self._keys.remove(key)

    def keys(self):
        return list(self._keys)

    def copy(self):
        copyDict = odict()
        copyDict._data = self._data.copy()
        copyDict._keys = self._keys[:]
        return copyDict


def compare_attr(key, params, strings):
    if not isinstance(strings, list):
        strings = [strings]
    if key in params:
        if params[key] == None:
            return False
        elif [str1 for str1 in strings if re.match(str1, params[key], re.IGNORECASE)]:
            return True
    return False

def is_attr(key, params, strings):
    if not isinstance(strings, list):
        strings = [strings]
    if key in params:
        if [str1 for str1 in strings if str1.lower() == params[key].lower()]:
            return True
    return False

def cleanout(dirty_string, delete_chars="", delete_words=[]):
    if not dirty_string:
        return dirty_string
    for dirt in delete_words + list(delete_chars):
        dirty_string = dirty_string.replace(dirt, "")
    return dirty_string.strip()

def substenv(matchobj):
    if matchobj.group(0).replace('%', '') in os.environ.keys():
        return os.environ[matchobj.group(0).replace('%', '')]
    else:
        return matchobj.group(0)

def normalize_dict(the_dict, titles=[]):
    for k in the_dict.keys():
        try:
            if k != k.lower():
                the_dict[k.lower()] = the_dict[k].strip()
                del the_dict[k]
            else:
                the_dict[k] = the_dict[k].strip()
        except Exception:
            pass
    for attr in [k for k in the_dict.keys() if k in titles]:
        try:
            the_dict[attr] = the_dict[attr].lower()
        except Exception:
            pass





import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "ROLE":
        return MonitoringDetailRole


class MonitoringDetailRole(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "role"
    property_type = str
    property_flat = True

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.role = params["monitoring_0"]

    def __str__(self):
        return "%s" % (self.role,)




#!/usr/bin/env python

# -*- coding: utf-8 -*-

import sys
try:
    python_version = sys.version_info
except:
    python_version = (1, 5)
if python_version < (2, 4):
    sys.exit("coshsh requires python 2.4.x and newer")
elif python_version >= (3,):
    sys.exit("coshsh requires python 2.x")

from setuptools import setup
from distutils import log
from distutils.command.install import install as _install
from distutils.command.install_data import install_data as _install_data
from distutils.core import setup, Command

import glob
import os

class CoshshTest(Command):
    user_options = []
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import subprocess
        import sys
        if 'PYTHONPATH' in os.environ:
            os.environ['PYTHONPATH'] += ":" + os.getcwd()
        else:
            os.environ['PYTHONPATH'] = os.getcwd()
        os.chdir('tests')
        for test in glob.glob('test_*.py'):
            log.info('Running test ' + test)
            errno = subprocess.call([sys.executable, test])
            if errno != 0:
                raise SystemExit(errno)


class install(_install):
    def run(self):
        _install.run(self)


class install_data(_install_data):
    def run(self):
        _install_data.run(self)


setup(name='coshsh',
      version='3.2.6',
      description='Coshsh - config generator for monitoring systems',
      url='http://github.com/lausser/coshsh',
      author='Gerhard Lausser',
      author_email='gerhard.lausser@consol.de',
      license='AGPLv3',
      packages=['coshsh'],
      data_files=[
          (os.path.join('recipes', 'default', 'classes'), 
              glob.glob(os.path.join('recipes', 'default', 'classes', '*.py'))),
          (os.path.join('recipes', 'default', 'templates'), 
              glob.glob(os.path.join('recipes', 'default', 'templates', '*.tpl')))
      ],
      scripts=['bin/coshsh-cook', 'bin/coshsh-create-template-tree'],
      cmdclass={'install_data': install_data, 'install': install, 'test': CoshshTest},
      zip_safe=False)




#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sys
import os
import re
import imp
import inspect
import logging
import coshsh
from coshsh.util import compare_attr, substenv

logger = logging.getLogger('coshsh')

class DatasourceNotImplemented(Exception):
    pass

class DatasourceNotReady(Exception):
    # datasource is currently being updated
    pass

class DatasourceNotCurrent(Exception):
    # datasources was not updated lately.
    # it makes no sense to continue.
    pass

class DatasourceNotAvailable(Exception):
    pass

class DatasourceCorrupt(Exception):
    pass


class Datasource(object):

    my_type = 'datasource'
    class_factory = []

    def __init__(self, **params):
        #print "datasourceinit with", self.__class__
        for key in [k for k in params if k.startswith("recipe_")]:
            setattr(self, key, params[key])
        for key in params.iterkeys():
            if isinstance(params[key], basestring):
                params[key] = re.sub('%.*?%', substenv, params[key])
        if self.__class__ == Datasource:
            #print "generic ds", params
            newcls = self.__class__.get_class(params)
            if newcls:
                #print "i rebless anon datasource to", newcls, params
                self.__class__ = newcls
                self.__init__(**params)
            else:
                logger.critical('datasource for %s is not implemented' % params)
                #print "i raise DatasourceNotImplemented"
                raise DatasourceNotImplemented
        else:
            setattr(self, 'name', params["name"])
            self.objects = {}
            pass
        # i am a generic datasource
        # i find a suitable class
        # i rebless
        # i call __init__
        
    def open(self, **kwargs):
        pass

    def read(self, **kwargs):
        pass

    def close(self):
        pass

    def add(self, objtype, obj):
        try:
            self.objects[objtype][obj.fingerprint()] = obj
        except Exception:
            self.objects[objtype] = {}
            self.objects[objtype][obj.fingerprint()] = obj

    def get(self, objtype, fingerprint):
        try:
            return self.objects[objtype][fingerprint]
        except Exception:
            # should be None
            return 'i do not exist. no. no!'

    def getall(self, objtype):
        try:
            return self.objects[objtype].values()
        except Exception:
            return []

    def find(self, objtype, fingerprint):
        return objtype in self.objects and fingerprint in self.objects[objtype]

    @classmethod
    def init_classes(cls, classpath):
        sys.dont_write_bytecode = True
        for p in [p for p in reversed(classpath) if os.path.exists(p) and os.path.isdir(p)]:
            for module, path in [(item, p) for item in os.listdir(p) if item[-3:] == ".py" and item.startswith('datasource_')]:
                try:
                    #print "try ds", module, path
                    path = os.path.abspath(path)
                    fp, filename, data = imp.find_module(module.replace('.py', ''), [path])
                    toplevel = imp.load_source(module.replace(".py", ""), filename)
                    for cl in inspect.getmembers(toplevel, inspect.isfunction):
                        if cl[0] ==  "__ds_ident__":
                            cls.class_factory.append([path, module, cl[1]])
                except Exception, exp:
                    logger.critical("could not load datasource %s from %s: %s" % (module, path, exp))
                finally:
                    if fp:
                        fp.close()


    @classmethod
    def get_class(cls, params={}):
        #print "get_classhoho", cls, len(cls.class_factory), cls.class_factory
        for path, module, class_func in cls.class_factory:
            try:
                #print "try", path, module, class_func
                newcls = class_func(params)
                if newcls:
                    return newcls
            except Exception ,exp:
                print "Datasource.get_class exception", exp
                pass
        logger.debug("found no matching class for this datasource %s" % params)







#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import re

# Jinja2 extensions
"""
** test re_match(pattern[, flags])
If regex pattern matches the string, return True.
Otherwise return False. The optional argument "flags"
is a string that may be empty (the default) or contain
one or more of regex flag characters.


*** filter re_sub(pattern, repl[, flags[, count]])
Return the string obtained by replacing the leftmost non-overlapping
occurrences of pattern in string by the replacement repl.
If the pattern isn't found, string is returned unchanged.


repl must be a string. Any backslash escapes in it are processed.
That is, \n is converted to a single newline character, \r is
converted
to a linefeed, and so forth. Unknown escapes such as \j are left
alone.
Backreferences, such as \6, are replaced with the substring
matched by group 6 in the pattern.


The optional argument "flags" is similar to the flags argument
in test re_match() (see above).


The optional argument count is the maximum number of pattern
occurrences to be replaced; count must be a non-negative integer.
If omitted or zero, all occurrences will be replaced.


*** filter re_escape
Return string with all non-alphanumerics backslashed;
this is useful if you want to match an arbitrary literal
string that may have regular expression metacharacters in it.
"""

def get_re_flags(flagstr):
    reflags = 0
    if flagstr:
        if flagstr.find("i") > -1: reflags |= re.IGNORECASE
        if flagstr.find("l") > -1: reflags |= re.LOCALE
        if flagstr.find("m") > -1: reflags |= re.MULTILINE
        if flagstr.find("s") > -1: reflags |= re.DOTALL
        if flagstr.find("u") > -1: reflags |= re.UNICODE
        if flagstr.find("x") > -1: reflags |= re.VERBOSE
    return reflags


def is_re_match(s, rs, flagstr=None):
    reflags = get_re_flags(flagstr)
    if re.search(rs, s, reflags):
        return True
    else:
        return False


def filter_re_sub(s, rs, repl, flagstr=None, count=0):
    reflags = get_re_flags(flagstr)
    myre = re.compile(rs, reflags)
    return re.sub(myre, repl, s, count)


def filter_re_escape(s):
    return re.escape(s)


# ['libkbo01.muc', 'os', 'linux', 'NAGIOSCONF', 'os_linux_default_check_ssh', 'max_check_attempts', '2', None, None, None],
# if a NAGIOSCONF attribute exists, the service is transformed into a
# service template (name=service_description+host_name) and a new service
# is created. the new service has all the NAGIOSCONF attributes and then
# uses the template
def filter_service(application, service_description):
    relevant_details = [d for d in application.monitoring_details if d.monitoring_type == "NAGIOSCONF" and d.name == service_description]
    if not relevant_details:
        snippet = "define service {\n  service_description             %s" % service_description
        if len(application.contact_groups) > 0:
            snippet += "\n  contact_groups %s" % (application.contact_groups, )
    else:
        snippet = "define service {\n  service_description             %s\n" % service_description
        raise "arsch"
        for detail in relevant_details:
            snippet += "  %-31s %s\n" % (detail.attribute, detail.value)
        snippet += "  use                             %s\n}\n" % (service_description + "_" + application.host_name, )
        snippet += "define service {\n  name                            %s\n" % (service_description + "_" + application.host_name, )
        snippet += "  register                        0"
        if len(application.contact_groups) > 0:
            snippet += "\n  contact_groups %s" % (application.contact_groups, )
    return snippet

def filter_custom_macros(application):
    snippet = ""
    for k, v in [x if x[0].startswith("_") else ("_" + x[0], x[1]) \
        for x in getattr(application, "custom_macros", {}).items() + \
                 getattr(application, "macros", {}).items()]:
        snippet += "  %-31s %s\n" % (k, v)
    return snippet



import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "DEPTH":
        return MonitoringDetailDepth


class MonitoringDetailDepth(coshsh.monitoringdetail.MonitoringDetail):
    """
    Describes hwo deep the monitoring of an application should go.
    Levels are 0 (ignore), 1 (minimum) ... n (expert level, debugging)
    """
    property = "monitoring_depth"
    property_type = int
    property_flat = True

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.monitoring_depth = int(params.get("monitoring_0", 1))

    def __str__(self):
        return "%s" % (self.monitoring_depth,)




import coshsh
from coshsh.application import Application
from coshsh.templaterule import TemplateRule
from coshsh.util import compare_attr

def __mi_ident__(params={}):
    if coshsh.util.compare_attr("type", params, ".*red\s*hat.*|.*rhel.*|.*sles.*|.*linux.*|.*limux.*|.*debian.*|.*ubuntu.*|.*centos.*"):
        return Linux


class Linux(coshsh.application.Application):
    template_rules = [
        coshsh.templaterule.TemplateRule(needsattr=None, 
            template="os_linux_default"),
        coshsh.templaterule.TemplateRule(needsattr="filesystems", 
            template="os_linux_fs"),
    ]

    def __new__(cls, params={}):
        if compare_attr("version", params, ".*embedded.*"):
            cls = EmbeddedLinux
        return object.__new__(cls)

    def __init__(self, params={}):
        self.swap_warning = "5%"
        self.swap_critical = "15%"
    


class RedHat(Linux):
    pass


class SuSE(Linux):
    pass


class EmbeddedLinux(Linux):
    template_rules = [
        coshsh.templaterule.TemplateRule(needsattr=None, 
            template="os_linux_heartbeat"),
    ]




from urlparse import urlparse
import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "URL":
        return MonitoringDetailUrl


class MonitoringDetailUrl(coshsh.monitoringdetail.MonitoringDetail):
    property = "urls"
    property_type = list

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.url = params.get("monitoring_0", None)
        self.warning = params.get("monitoring_1", "5") or "5"
        self.critical = params.get("monitoring_2", "10") or "10"
        self.url_expect = params.get("monitoring_3", None)
        scheme, rest = self.url.split(":", 1)
        o = urlparse("http:" + rest)
        for attr in ["scheme", "netloc", "path", "params", "query", "fragment", "username", "password", "hostname", "port"]:
            setattr(self, attr, getattr(o, attr))
        setattr(self, "scheme", scheme)

    def __str__(self):
        return "%s %s:%s" % (self.url, self.warning, self.critical)




import coshsh
from coshsh.monitoringdetail import MonitoringDetail

def __detail_ident__(params={}):
    if params["monitoring_type"] == "DATASTORE":
        return MonitoringDetailDatastore


class MonitoringDetailDatastore(coshsh.monitoringdetail.MonitoringDetail):
    """
    """
    property = "datastores"
    property_type = list

    def __init__(self, params):
        self.monitoring_type = params["monitoring_type"]
        self.name = params["monitoring_0"]
        self.path = "/vmfs/volumes/%s" % self.name
        self.warning = params.get("monitoring_1", "10")
        self.critical = params.get("monitoring_2", "5")
        self.units = params.get("monitoring_3", "%")

    def __str__(self):
        return "%s %s:%s:0:100" % (self.name, self.warning, self.critical)





#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#
# This file belongs to coshsh.
# Copyright Gerhard Lausser.
# This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sys
import os
import imp
import inspect
import logging
import coshsh
from coshsh.util import compare_attr, is_attr
from coshsh.item import Item
from coshsh.templaterule import TemplateRule

logger = logging.getLogger('coshsh')

class ApplicationNotImplemented(Exception):
    pass


class Application(coshsh.item.Item):

    id = 1 #0 is reserved for host (primary node for parents)
    my_type = 'application'
    app_template = "app.tpl"
    class_factory = []
    lower_columns = ['name', 'type', 'component', 'version', 'patchlevel']


    def __init__(self, params):
        #print "Application init", self.__class__, self.__class__.__name__, len(self.__class__.class_factory)
        if self.__class__.__name__ == "Application":
            for c in self.__class__.lower_columns:
                try:
                    params[c] = params[c].lower()
                except Exception:
                    if c in params:
                        params[c] = None
            newcls = self.__class__.get_class(params)
            if newcls:
                self.__class__ = newcls
                self.contact_groups = []
                super(Application, self).__init__(params)
                self.__init__(params)
                self.fingerprint = lambda s=self:s.__class__.fingerprint(params)
            else:
                logger.debug('this will be Generic %s' % params)
                self.__class__ = GenericApplication
                self.contact_groups = []
                super(Application, self).__init__(params)
                self.__init__(params)
                #raise ApplicationNotImplemented
                self.fingerprint = lambda s=self:s.__class__.fingerprint(params)
        else:
            pass

    @classmethod
    def fingerprint(self, params={}):
        return "%s+%s+%s" % (params["host_name"], params["name"], params["type"])

    def _i_init__(self, params={}):
        super(Application, self).__init__(params)
        self.contact_groups = []

    def create_servicegroups(self):
        pass

    def create_contacts(self):
        pass

    def create_templates(self):
        pass

    @classmethod
    def init_classes(cls, classpath):
        sys.dont_write_bytecode = True
        for p in [p for p in reversed(classpath) if os.path.exists(p) and os.path.isdir(p)]:
            for module, path in [(item, p) for item in os.listdir(p) if item[-3:] == ".py" and (item.startswith('app_') or item.startswith('os_'))]:
                try:
                    path = os.path.abspath(path)
                    fp, filename, data = imp.find_module(module.replace('.py', ''), [path])
                    toplevel = imp.load_source(module.replace(".py", ""), filename)
                    for cl in inspect.getmembers(toplevel, inspect.isfunction):
                        if cl[0] ==  "__mi_ident__":
                            cls.class_factory.append([path, module, cl[1]])
                except Exception, e:
                    print e
                finally:
                    if fp:
                        fp.close()
        #print ".............fill %s / %s woth %s" % (cls, cls.__name__, cls.class_factory)


    @classmethod
    def get_class(cls, params={}):
        #print "getclass from cache", cls, cls.__name__,  cls.class_factory
        for path, module, class_func in cls.class_factory:
            try:
                #print "get_class trys", path, module, class_func
                newcls = class_func(params)
                #print "get_class says", newcls
                if newcls:
                    return newcls
            except Exception:
                pass
        logger.debug("found no matching class for this monitoring item %s" % params)
        return None


class GenericApplication(Application):
    template_rules = [
        TemplateRule(needsattr=None, 
            template="app_generic_default",
            unique_attr=['type', 'name'], unique_config="app_%s_%s_default"),
    ]

    def x__new__(cls, params={}):
        return object.__new__(cls)

    def __init__(self, params={}):
        self.name = params["name"]
        super(GenericApplication, self).__init__(params)

    def render(self, template_cache, jinja2):
        # Maybe we find some processes, ports, filesystems in the
        # monitoring_details so we can output generic services
        if (hasattr(self, "processes") and self.processes) or (hasattr(self, "filesystems") and self.filesystems) or (hasattr(self, "cfgfiles") and self.cfgfiles) or (hasattr(self, "files") and self.files) or (hasattr(self, "ports") and self.ports) or (hasattr(self, "urls") and self.urls) or (hasattr(self, "services") and self.services):
            super(GenericApplication, self).render(template_cache, jinja2)
        else:
            return ()


